import SpectrumCommon.Const.GB as gbc

import SpectrumCommon.Const.PC.PCConst as pcc
import SpectrumCommon.Const.PC.PCConstLink as pccl

def get_PC_row_curr_ID(mst_ID):
    curr_ID = 0

    # Standard rows
    if mst_ID == pcc.PC_PROG_SPEC_HR_HEADER_ROW_MST_ID:
        curr_ID = pcc.PC_PROG_SPEC_HR_HEADER_ROW_CURR_ID

    elif mst_ID == pcc.PC_NAT_LEVEL_STAFF_ROW_MST_ID:
        curr_ID = pcc.PC_NAT_LEVEL_STAFF_ROW_CURR_ID

    elif mst_ID == pcc.PC_REG_LEVEL_STAFF_ROW_MST_ID:
        curr_ID = pcc.PC_REG_LEVEL_STAFF_ROW_CURR_ID

    elif mst_ID == pcc.PC_DIST_LEVEL_STAFF_ROW_MST_ID:
        curr_ID = pcc.PC_DIST_LEVEL_STAFF_ROW_CURR_ID

    elif mst_ID == pcc.PC_TRAIN_HEADER_ROW_MST_ID:
        curr_ID = pcc.PC_TRAIN_HEADER_ROW_CURR_ID

    elif mst_ID == pcc.PC_INSERV_REFRESH_TRAIN_ROW_MST_ID:
        curr_ID = pcc.PC_INSERV_REFRESH_TRAIN_ROW_CURR_ID

    elif mst_ID == pcc.PC_TRAIN_OF_TRAIN_ROW_MST_ID:
        curr_ID = pcc.PC_TRAIN_OF_TRAIN_ROW_CURR_ID

    elif mst_ID == pcc.PC_DEV_TRAIN_PROG_MAT_ROW_MST_ID:
        curr_ID = pcc.PC_DEV_TRAIN_PROG_MAT_ROW_CURR_ID

    elif mst_ID == pcc.PC_CHANGE_PRESERV_TRAIN_CURRIC_ROW_MST_ID:
        curr_ID = pcc.PC_CHANGE_PRESERV_TRAIN_CURRIC_ROW_CURR_ID

    elif mst_ID == pcc.PC_SUPP_ACT_ROW_MST_ID:
        curr_ID = pcc.PC_SUPP_ACT_ROW_CURR_ID

    elif mst_ID == pcc.PC_SUPERVIS_HEADER_ROW_MST_ID:
        curr_ID = pcc.PC_SUPERVIS_HEADER_ROW_CURR_ID

    elif mst_ID == pcc.PC_COORD_MEET_ROW_MST_ID:
        curr_ID = pcc.PC_COORD_MEET_ROW_CURR_ID

    elif mst_ID == pcc.PC_NAT_STAFF_VISIT_LOCAL_ROW_MST_ID:
        curr_ID = pcc.PC_NAT_STAFF_VISIT_LOCAL_ROW_CURR_ID

    elif mst_ID == pcc.PC_MONITOR_EVAL_HEADER_ROW_MST_ID:
        curr_ID = pcc.PC_MONITOR_EVAL_HEADER_ROW_CURR_ID

    elif mst_ID == pcc.PC_DESIGN_ME_FRAME_SYS_ROW_MST_ID:
        curr_ID = pcc.PC_DESIGN_ME_FRAME_SYS_ROW_CURR_ID

    elif mst_ID == pcc.PC_DESIGN_QC_QA_ROW_MST_ID:
        curr_ID = pcc.PC_DESIGN_QC_QA_ROW_CURR_ID

    elif mst_ID == pcc.PC_DESIGN_REV_DATA_MGMT_SYS_ROW_MST_ID:
        curr_ID = pcc.PC_DESIGN_REV_DATA_MGMT_SYS_ROW_CURR_ID
 
    elif mst_ID == pcc.PC_DATA_COLLECT_ANALYSIS_ROW_MST_ID:
        curr_ID = pcc.PC_DATA_COLLECT_ANALYSIS_ROW_CURR_ID

    elif mst_ID == pcc.PC_QC_QA_ROW_MST_ID:
        curr_ID = pcc.PC_QC_QA_ROW_CURR_ID

    elif mst_ID == pcc.PC_INFRA_EQUIP_HEADER_ROW_MST_ID:
        curr_ID = pcc.PC_INFRA_EQUIP_HEADER_ROW_CURR_ID

    elif mst_ID == pcc.PC_SIT_ASSESS_INFRA_ROW_MST_ID:
        curr_ID = pcc.PC_SIT_ASSESS_INFRA_ROW_CURR_ID

    elif mst_ID == pcc.PC_EQUIP_UPGRADES_LOW_TIER_ROW_MST_ID:
        curr_ID = pcc.PC_EQUIP_UPGRADES_LOW_TIER_ROW_CURR_ID

    elif mst_ID == pcc.PC_EQUIP_UPGRADES_HOSP_ROW_MST_ID:
        curr_ID = pcc.PC_EQUIP_UPGRADES_HOSP_ROW_CURR_ID

    elif mst_ID == pcc.PC_TRANSPORT_HEADER_ROW_MST_ID:
        curr_ID = pcc.PC_TRANSPORT_HEADER_ROW_CURR_ID

    elif mst_ID == pcc.PC_SIT_ASSESS_TRANSPORT_ROW_MST_ID:
        curr_ID = pcc.PC_SIT_ASSESS_TRANSPORT_ROW_CURR_ID

    elif mst_ID == pcc.PC_NEW_VEHIC_PURCHASE_ROW_MST_ID:
        curr_ID = pcc.PC_NEW_VEHIC_PURCHASE_ROW_CURR_ID

    elif mst_ID == pcc.PC_VEHIC_OP_MAINT_ROW_MST_ID:
        curr_ID = pcc.PC_VEHIC_OP_MAINT_ROW_CURR_ID

    elif mst_ID == pcc.PC_COMM_MEDIA_HEADER_ROW_MST_ID:
        curr_ID = pcc.PC_COMM_MEDIA_HEADER_ROW_CURR_ID

    elif mst_ID == pcc.PC_DEV_COMM_STRAT_ROW_MST_ID:
        curr_ID = pcc.PC_DEV_COMM_STRAT_ROW_CURR_ID

    elif mst_ID == pcc.PC_MASS_MEDIA_COMM_ROW_MST_ID:
        curr_ID = pcc.PC_MASS_MEDIA_COMM_ROW_CURR_ID

    elif mst_ID == pcc.PC_PRINTED_MATS_ROW_MST_ID:
        curr_ID = pcc.PC_PRINTED_MATS_ROW_CURR_ID

    elif mst_ID == pcc.PC_SOCIAL_OUTR_ACTIV_ROW_MST_ID:
        curr_ID = pcc.PC_SOCIAL_OUTR_ACTIV_ROW_CURR_ID

    elif mst_ID == pcc.PC_ADVOC_HEADER_ROW_MST_ID:
        curr_ID = pcc.PC_ADVOC_HEADER_ROW_CURR_ID

    elif mst_ID == pcc.PC_PLAN_ADVOC_STRAT_ROW_MST_ID:
        curr_ID = pcc.PC_PLAN_ADVOC_STRAT_ROW_CURR_ID

    elif mst_ID == pcc.PC_ADVOC_ACTS_ROW_MST_ID:
        curr_ID = pcc.PC_ADVOC_ACTS_ROW_CURR_ID

    elif mst_ID == pcc.PC_ADVOC_MATS_ROW_MST_ID:
        curr_ID = pcc.PC_ADVOC_MATS_ROW_CURR_ID

    elif mst_ID == pcc.PC_GEN_PROG_MGMT_HEADER_ROW_MST_ID:
        curr_ID = pcc.PC_GEN_PROG_MGMT_HEADER_ROW_CURR_ID

    elif mst_ID == pcc.PC_DESIGN_REV_COUNTRY_STRAT_ROW_MST_ID:
        curr_ID = pcc.PC_DESIGN_REV_COUNTRY_STRAT_ROW_CURR_ID

    elif mst_ID == pcc.PC_DEV_REVIEW_ANNUAL_WORKPLAN_ROW_MST_ID:
        curr_ID = pcc.PC_DEV_REVIEW_ANNUAL_WORKPLAN_ROW_CURR_ID

    elif mst_ID == pcc.PC_DEV_REVIEW_HR_PLAN_ROW_MST_ID:
        curr_ID = pcc.PC_DEV_REVIEW_HR_PLAN_ROW_CURR_ID

    elif mst_ID == pcc.PC_PROG_COORD_MEET_ROW_MST_ID:
        curr_ID = pcc.PC_PROG_COORD_MEET_ROW_CURR_ID

    elif mst_ID == pcc.PC_COMMOD_REG_POL_ROW_MST_ID:
        curr_ID = pcc.PC_COMMOD_REG_POL_ROW_CURR_ID

    elif mst_ID == pcc.PC_SIT_ANALYSIS_ROW_MST_ID:
        curr_ID = pcc.PC_SIT_ANALYSIS_ROW_CURR_ID

    elif mst_ID == pcc.PC_OTHER_HEADER_ROW_MST_ID:
        curr_ID = pcc.PC_OFFICE_EQUIP_SUPP_ROW_CURR_ID

    elif mst_ID == pcc.PC_DBL_CLICK_ADD_OTHER_ITEMS_ROW_MST_ID:
        curr_ID = pcc.PC_DBL_CLICK_ADD_OTHER_ITEMS_ROW_CURR_ID 

    elif mst_ID == pcc.PC_OFFICE_EQUIP_SUPP_ROW_MST_ID:
        curr_ID = pcc.PC_OTHER_HEADER_ROW_CURR_ID

    elif mst_ID == pcc.PC_UTIL_ROW_MST_ID:
        curr_ID = pcc.PC_UTIL_ROW_CURR_ID

    # HIV rows

    elif mst_ID == pcc.PC_POLICY_BASED_INTERV_ROW_MST_ID:
        curr_ID = pcc.PC_POLICY_BASED_INTERV_ROW_CURR_ID

    elif mst_ID == pcc.PC_SOC_ECON_SUPP_CHILD_ROW_MST_ID:
        curr_ID = pcc.PC_SOC_ECON_SUPP_CHILD_ROW_CURR_ID

    elif mst_ID == pcc.PC_PREV_VIOL_WOMEN_ROW_MST_ID: 
        curr_ID = pcc.PC_PREV_VIOL_WOMEN_ROW_CURR_ID

    elif mst_ID == pcc.PC_DECRIM_KEY_POP_BEHAV_ROW_MST_ID:
        curr_ID = pcc.PC_DECRIM_KEY_POP_BEHAV_ROW_CURR_ID

    elif mst_ID == pcc.PC_PREV_STIGMA_DISCRIM_ROW_MST_ID:
        curr_ID = pcc.PC_PREV_STIGMA_DISCRIM_ROW_CURR_ID

    elif mst_ID == pcc.PC_COMM_CIVIL_SOC_LEAD_ROW_MST_ID:
        curr_ID = pcc.PC_COMM_CIVIL_SOC_LEAD_ROW_CURR_ID

    elif mst_ID == pcc.PC_LEGAL_POLICY_REFORMS_ROW_MST_ID:
        curr_ID = pcc.PC_LEGAL_POLICY_REFORMS_ROW_CURR_ID

    elif mst_ID == pcc.PC_HIV_OTHER_HEADER_ROW_MST_ID:
        curr_ID = pcc.PC_HIV_OTHER_HEADER_ROW_CURR_ID

    elif mst_ID == pcc.PC_HIV_DBL_CLICK_ADD_OTHER_ITEMS_ROW_MST_ID:
        curr_ID = pcc.PC_HIV_DBL_CLICK_ADD_OTHER_ITEMS_ROW_CURR_ID

    return curr_ID

# Need the programme area ID because some programme areas have different rows and the current IDs can overlap, unlike the master IDs.
def get_PC_row_mst_ID(prog_area_ID: str, curr_ID: int):
    mst_ID = 0

    if (prog_area_ID == pccl.PC_IH_HIV_AIDS_MST_ID) and (curr_ID >= pcc.PC_POLICY_BASED_INTERV_ROW_CURR_ID):

        if curr_ID == pcc.PC_POLICY_BASED_INTERV_ROW_CURR_ID:
            mst_ID = pcc.PC_POLICY_BASED_INTERV_ROW_MST_ID

        elif curr_ID == pcc.PC_SOC_ECON_SUPP_CHILD_ROW_CURR_ID:
            mst_ID = pcc.PC_SOC_ECON_SUPP_CHILD_ROW_MST_ID

        elif curr_ID == pcc.PC_PREV_VIOL_WOMEN_ROW_CURR_ID: 
            mst_ID = pcc.PC_PREV_VIOL_WOMEN_ROW_MST_ID

        elif curr_ID == pcc.PC_DECRIM_KEY_POP_BEHAV_ROW_CURR_ID:
            mst_ID = pcc.PC_DECRIM_KEY_POP_BEHAV_ROW_MST_ID

        elif curr_ID == pcc.PC_PREV_STIGMA_DISCRIM_ROW_CURR_ID:
            mst_ID = pcc.PC_PREV_STIGMA_DISCRIM_ROW_MST_ID

        elif curr_ID == pcc.PC_COMM_CIVIL_SOC_LEAD_ROW_CURR_ID:
            mst_ID = pcc.PC_COMM_CIVIL_SOC_LEAD_ROW_MST_ID

        elif curr_ID == pcc.PC_LEGAL_POLICY_REFORMS_ROW_CURR_ID:
            mst_ID = pcc.PC_LEGAL_POLICY_REFORMS_ROW_MST_ID

        elif curr_ID == pcc.PC_HIV_OTHER_HEADER_ROW_CURR_ID:
            mst_ID = pcc.PC_HIV_OTHER_HEADER_ROW_MST_ID           

        elif curr_ID == pcc.PC_HIV_DBL_CLICK_ADD_OTHER_ITEMS_ROW_CURR_ID:
            mst_ID = pcc.PC_HIV_DBL_CLICK_ADD_OTHER_ITEMS_ROW_MST_ID

    elif (prog_area_ID != pccl.PC_IH_HIV_AIDS_MST_ID) or (curr_ID < pcc.PC_SOC_ECON_SUPP_CHILD_ROW_CURR_ID):

        if curr_ID == pcc.PC_PROG_SPEC_HR_HEADER_ROW_CURR_ID:
            mst_ID = pcc.PC_PROG_SPEC_HR_HEADER_ROW_MST_ID

        elif curr_ID == pcc.PC_NAT_LEVEL_STAFF_ROW_CURR_ID:
            mst_ID = pcc.PC_NAT_LEVEL_STAFF_ROW_MST_ID

        elif curr_ID == pcc.PC_REG_LEVEL_STAFF_ROW_CURR_ID:
            mst_ID = pcc.PC_REG_LEVEL_STAFF_ROW_MST_ID

        elif curr_ID == pcc.PC_DIST_LEVEL_STAFF_ROW_CURR_ID:
            mst_ID = pcc.PC_DIST_LEVEL_STAFF_ROW_MST_ID

        elif curr_ID == pcc.PC_TRAIN_HEADER_ROW_CURR_ID:
            mst_ID = pcc.PC_TRAIN_HEADER_ROW_MST_ID

        elif curr_ID == pcc.PC_INSERV_REFRESH_TRAIN_ROW_CURR_ID:
            mst_ID = pcc.PC_INSERV_REFRESH_TRAIN_ROW_MST_ID

        elif curr_ID == pcc.PC_TRAIN_OF_TRAIN_ROW_CURR_ID:
            mst_ID = pcc.PC_TRAIN_OF_TRAIN_ROW_MST_ID

        elif curr_ID == pcc.PC_DEV_TRAIN_PROG_MAT_ROW_CURR_ID:
            mst_ID = pcc.PC_DEV_TRAIN_PROG_MAT_ROW_MST_ID

        elif curr_ID == pcc.PC_CHANGE_PRESERV_TRAIN_CURRIC_ROW_CURR_ID:
            mst_ID = pcc.PC_CHANGE_PRESERV_TRAIN_CURRIC_ROW_MST_ID

        elif curr_ID == pcc.PC_SUPP_ACT_ROW_CURR_ID:
            mst_ID = pcc.PC_SUPP_ACT_ROW_MST_ID

        elif curr_ID == pcc.PC_SUPERVIS_HEADER_ROW_CURR_ID:
            mst_ID = pcc.PC_SUPERVIS_HEADER_ROW_MST_ID

        elif curr_ID == pcc.PC_COORD_MEET_ROW_CURR_ID:
            mst_ID = pcc.PC_COORD_MEET_ROW_MST_ID

        elif curr_ID == pcc.PC_NAT_STAFF_VISIT_LOCAL_ROW_CURR_ID:
            mst_ID = pcc.PC_NAT_STAFF_VISIT_LOCAL_ROW_MST_ID

        elif curr_ID == pcc.PC_MONITOR_EVAL_HEADER_ROW_CURR_ID:
            mst_ID = pcc.PC_MONITOR_EVAL_HEADER_ROW_MST_ID

        elif curr_ID == pcc.PC_DESIGN_ME_FRAME_SYS_ROW_CURR_ID:
            mst_ID = pcc.PC_DESIGN_ME_FRAME_SYS_ROW_MST_ID

        elif curr_ID == pcc.PC_DESIGN_QC_QA_ROW_CURR_ID:
            mst_ID = pcc.PC_DESIGN_QC_QA_ROW_MST_ID

        elif curr_ID == pcc.PC_DESIGN_REV_DATA_MGMT_SYS_ROW_CURR_ID:
            mst_ID = pcc.PC_DESIGN_REV_DATA_MGMT_SYS_ROW_MST_ID
    
        elif curr_ID == pcc.PC_DATA_COLLECT_ANALYSIS_ROW_CURR_ID:
            mst_ID = pcc.PC_DATA_COLLECT_ANALYSIS_ROW_MST_ID

        elif curr_ID == pcc.PC_QC_QA_ROW_CURR_ID:
            mst_ID = pcc.PC_QC_QA_ROW_MST_ID

        elif curr_ID == pcc.PC_INFRA_EQUIP_HEADER_ROW_CURR_ID:
            mst_ID = pcc.PC_INFRA_EQUIP_HEADER_ROW_MST_ID

        elif curr_ID == pcc.PC_SIT_ASSESS_INFRA_ROW_CURR_ID:
            mst_ID = pcc.PC_SIT_ASSESS_INFRA_ROW_MST_ID

        elif curr_ID == pcc.PC_EQUIP_UPGRADES_LOW_TIER_ROW_CURR_ID:
            mst_ID = pcc.PC_EQUIP_UPGRADES_LOW_TIER_ROW_MST_ID

        elif curr_ID == pcc.PC_EQUIP_UPGRADES_HOSP_ROW_CURR_ID:
            mst_ID = pcc.PC_EQUIP_UPGRADES_HOSP_ROW_MST_ID

        elif curr_ID == pcc.PC_TRANSPORT_HEADER_ROW_CURR_ID:
            mst_ID = pcc.PC_TRANSPORT_HEADER_ROW_MST_ID

        elif curr_ID == pcc.PC_SIT_ASSESS_TRANSPORT_ROW_CURR_ID:
            mst_ID = pcc.PC_SIT_ASSESS_TRANSPORT_ROW_MST_ID

        elif curr_ID == pcc.PC_NEW_VEHIC_PURCHASE_ROW_CURR_ID:
            mst_ID = pcc.PC_NEW_VEHIC_PURCHASE_ROW_MST_ID

        elif curr_ID == pcc.PC_VEHIC_OP_MAINT_ROW_CURR_ID:
            mst_ID = pcc.PC_VEHIC_OP_MAINT_ROW_MST_ID

        elif curr_ID == pcc.PC_COMM_MEDIA_HEADER_ROW_CURR_ID:
            mst_ID = pcc.PC_COMM_MEDIA_HEADER_ROW_MST_ID

        elif curr_ID == pcc.PC_DEV_COMM_STRAT_ROW_CURR_ID:
            mst_ID = pcc.PC_DEV_COMM_STRAT_ROW_MST_ID

        elif curr_ID == pcc.PC_MASS_MEDIA_COMM_ROW_CURR_ID:
            mst_ID = pcc.PC_MASS_MEDIA_COMM_ROW_MST_ID

        elif curr_ID == pcc.PC_PRINTED_MATS_ROW_CURR_ID:
            mst_ID = pcc.PC_PRINTED_MATS_ROW_MST_ID

        elif curr_ID == pcc.PC_SOCIAL_OUTR_ACTIV_ROW_CURR_ID:
            mst_ID = pcc.PC_SOCIAL_OUTR_ACTIV_ROW_MST_ID

        elif curr_ID == pcc.PC_ADVOC_HEADER_ROW_CURR_ID:
            mst_ID = pcc.PC_ADVOC_HEADER_ROW_MST_ID

        elif curr_ID == pcc.PC_PLAN_ADVOC_STRAT_ROW_CURR_ID:
            mst_ID = pcc.PC_PLAN_ADVOC_STRAT_ROW_MST_ID

        elif curr_ID == pcc.PC_ADVOC_ACTS_ROW_CURR_ID:
            mst_ID = pcc.PC_ADVOC_ACTS_ROW_MST_ID

        elif curr_ID == pcc.PC_ADVOC_MATS_ROW_CURR_ID:
            mst_ID = pcc.PC_ADVOC_MATS_ROW_MST_ID

        elif curr_ID == pcc.PC_GEN_PROG_MGMT_HEADER_ROW_CURR_ID:
            mst_ID = pcc.PC_GEN_PROG_MGMT_HEADER_ROW_MST_ID

        elif curr_ID == pcc.PC_DESIGN_REV_COUNTRY_STRAT_ROW_CURR_ID:
            mst_ID = pcc.PC_DESIGN_REV_COUNTRY_STRAT_ROW_MST_ID

        elif curr_ID == pcc.PC_DEV_REVIEW_ANNUAL_WORKPLAN_ROW_CURR_ID:
            mst_ID = pcc.PC_DEV_REVIEW_ANNUAL_WORKPLAN_ROW_MST_ID

        elif curr_ID == pcc.PC_DEV_REVIEW_HR_PLAN_ROW_CURR_ID:
            mst_ID = pcc.PC_DEV_REVIEW_HR_PLAN_ROW_MST_ID

        elif curr_ID == pcc.PC_PROG_COORD_MEET_ROW_CURR_ID:
            mst_ID = pcc.PC_PROG_COORD_MEET_ROW_MST_ID

        elif curr_ID == pcc.PC_COMMOD_REG_POL_ROW_CURR_ID:
            mst_ID = pcc.PC_COMMOD_REG_POL_ROW_MST_ID

        elif curr_ID == pcc.PC_SIT_ANALYSIS_ROW_CURR_ID:
            mst_ID = pcc.PC_SIT_ANALYSIS_ROW_MST_ID

        elif curr_ID == pcc.PC_OTHER_HEADER_ROW_CURR_ID:
            mst_ID = pcc.PC_OTHER_HEADER_ROW_MST_ID

        elif curr_ID == pcc.PC_DBL_CLICK_ADD_OTHER_ITEMS_ROW_CURR_ID:
            mst_ID = pcc.PC_DBL_CLICK_ADD_OTHER_ITEMS_ROW_MST_ID 

        elif curr_ID == pcc.PC_OFFICE_EQUIP_SUPP_ROW_CURR_ID:
            mst_ID = pcc.PC_OFFICE_EQUIP_SUPP_ROW_MST_ID

        elif curr_ID == pcc.PC_UTIL_ROW_CURR_ID:
            mst_ID = pcc.PC_UTIL_ROW_MST_ID

    return mst_ID

def get_PC_row_name(mst_ID):
    name = ''

    # Standard rows

    if mst_ID == pcc.PC_PROG_SPEC_HR_HEADER_ROW_MST_ID:
        name = '1. Programme-specific human resources'

    elif mst_ID == pcc.PC_NAT_LEVEL_STAFF_ROW_MST_ID:
        name = '1.1 National-level staff'

    elif mst_ID == pcc.PC_REG_LEVEL_STAFF_ROW_MST_ID:
        name = '1.2 Regional-level staff'

    elif mst_ID == pcc.PC_DIST_LEVEL_STAFF_ROW_MST_ID:
        name = '1.3 District-level staff'

    elif mst_ID == pcc.PC_TRAIN_HEADER_ROW_MST_ID:
        name = '2. Training'

    elif mst_ID == pcc.PC_INSERV_REFRESH_TRAIN_ROW_MST_ID:
        name = '2.1 In-service / Refresher training'

    elif mst_ID == pcc.PC_TRAIN_OF_TRAIN_ROW_MST_ID:
        name = '2.2 Training of trainers'

    elif mst_ID == pcc.PC_DEV_TRAIN_PROG_MAT_ROW_MST_ID:
        name = '2.3 Development of training programmes and material'

    elif mst_ID == pcc.PC_CHANGE_PRESERV_TRAIN_CURRIC_ROW_MST_ID:
        name = '2.4 Changing the pre-service training curriculum'

    elif mst_ID == pcc.PC_SUPP_ACT_ROW_MST_ID:
        name = '2.5 Support activities'

    elif mst_ID == pcc.PC_SUPERVIS_HEADER_ROW_MST_ID:
        name = '3. Supervision'

    elif mst_ID == pcc.PC_COORD_MEET_ROW_MST_ID:
        name = '3.1 Coordination meetings'

    elif mst_ID == pcc.PC_NAT_STAFF_VISIT_LOCAL_ROW_MST_ID:
        name = '3.2 National staff visiting local staff'

    elif mst_ID == pcc.PC_MONITOR_EVAL_HEADER_ROW_MST_ID:
        name = '4. Monitoring and evaluation'

    elif mst_ID == pcc.PC_DESIGN_ME_FRAME_SYS_ROW_MST_ID:
        name = '4.1 Design of M & E frameworks and systems'

    elif mst_ID == pcc.PC_DESIGN_QC_QA_ROW_MST_ID:
        name = '4.2 Design of quality control and assurance'

    elif mst_ID == pcc.PC_DESIGN_REV_DATA_MGMT_SYS_ROW_MST_ID:
        name = '4.3 Design / review of data management systems'
 
    elif mst_ID == pcc.PC_DATA_COLLECT_ANALYSIS_ROW_MST_ID:
        name = '4.4 Data collection and analysis'

    elif mst_ID == pcc.PC_QC_QA_ROW_MST_ID:
        name = '4.5 Quality control / quality assurance'

    elif mst_ID == pcc.PC_INFRA_EQUIP_HEADER_ROW_MST_ID:
        name = '5. Infrastructure and equipment'

    elif mst_ID == pcc.PC_SIT_ASSESS_INFRA_ROW_MST_ID:
        name = '5.1  Situational assessment'

    elif mst_ID == pcc.PC_EQUIP_UPGRADES_LOW_TIER_ROW_MST_ID:
        name = '5.2 Equipment upgrades for lower-tier facilities'

    elif mst_ID == pcc.PC_EQUIP_UPGRADES_HOSP_ROW_MST_ID:
        name = '5.3 Equipment upgrades for hospitals'

    elif mst_ID == pcc.PC_TRANSPORT_HEADER_ROW_MST_ID:
        name = '6. Transport'

    elif mst_ID == pcc.PC_SIT_ASSESS_TRANSPORT_ROW_MST_ID:
        name = '6.1 Situational assessment'

    elif mst_ID == pcc.PC_NEW_VEHIC_PURCHASE_ROW_MST_ID:
        name = '6.2 New vehicle purchase'

    elif mst_ID == pcc.PC_VEHIC_OP_MAINT_ROW_MST_ID:
        name = '6.3 Vehicle operation and maintenance'

    elif mst_ID == pcc.PC_COMM_MEDIA_HEADER_ROW_MST_ID:
        name = '7. Communication, media, and outreach'

    elif mst_ID == pcc.PC_DEV_COMM_STRAT_ROW_MST_ID:
        name = '7.1 Development of communication strategy'

    elif mst_ID == pcc.PC_MASS_MEDIA_COMM_ROW_MST_ID:
        name = '7.2 Mass media'

    elif mst_ID == pcc.PC_PRINTED_MATS_ROW_MST_ID:
        name = '7.3 Printed materials'

    elif mst_ID == pcc.PC_SOCIAL_OUTR_ACTIV_ROW_MST_ID:
        name = '7.4 Social outreach activities'

    elif mst_ID == pcc.PC_ADVOC_HEADER_ROW_MST_ID:
        name = '8. Advocacy'

    elif mst_ID == pcc.PC_PLAN_ADVOC_STRAT_ROW_MST_ID:
        name = '8.1 Planning on advocacy strategy'

    elif mst_ID == pcc.PC_ADVOC_ACTS_ROW_MST_ID:
        name = '8.2 Advocacy activities'

    elif mst_ID == pcc.PC_ADVOC_MATS_ROW_MST_ID:
        name = '8.3 Advocacy materials'

    elif mst_ID == pcc.PC_GEN_PROG_MGMT_HEADER_ROW_MST_ID:
        name = '9. General programme management'

    elif mst_ID == pcc.PC_DESIGN_REV_COUNTRY_STRAT_ROW_MST_ID:
        name = '9.1 Design and review of country strategy'

    elif mst_ID == pcc.PC_DEV_REVIEW_ANNUAL_WORKPLAN_ROW_MST_ID:
        name = '9.2 Development and review of annual work plan'

    elif mst_ID == pcc.PC_DEV_REVIEW_HR_PLAN_ROW_MST_ID:
        name = '9.3 Development and review of human resources plan'

    elif mst_ID == pcc.PC_PROG_COORD_MEET_ROW_MST_ID:
        name = '9.4 Programme coordination meetings'

    elif mst_ID == pcc.PC_COMMOD_REG_POL_ROW_MST_ID:
        name = '9.5 Commodity regulation and policies'

    elif mst_ID == pcc.PC_SIT_ANALYSIS_ROW_MST_ID:
        name = '9.6 Situation analysis'

    elif mst_ID == pcc.PC_OFFICE_EQUIP_SUPP_ROW_MST_ID:
        name = '9.7 Office equipment and supplies'
        
    elif mst_ID == pcc.PC_UTIL_ROW_MST_ID:
        name = '9.8 Utilities'

    elif mst_ID == pcc.PC_OTHER_HEADER_ROW_MST_ID:
        name = '10. Other'

    elif mst_ID == pcc.PC_DBL_CLICK_ADD_OTHER_ITEMS_ROW_MST_ID:
        name = '10.1 Other items'

    # HIV-specific rows

    elif mst_ID == pcc.PC_POLICY_BASED_INTERV_ROW_MST_ID:
        name = '10. Policy-based interventions'      

    elif mst_ID == pcc.PC_SOC_ECON_SUPP_CHILD_ROW_MST_ID:
        name = '10.1 Socioeconomic support for orphans and vulnerable children'         

    elif mst_ID == pcc.PC_PREV_VIOL_WOMEN_ROW_MST_ID:           
        name = '10.2 Prevention of violence against women'    

    elif mst_ID == pcc.PC_DECRIM_KEY_POP_BEHAV_ROW_MST_ID:      
        name = '10.3 Decriminalization of key population behaviors'   

    elif mst_ID == pcc.PC_PREV_STIGMA_DISCRIM_ROW_MST_ID:       
        name = '10.4 Prevention of stigma and discrimination'    

    elif mst_ID == pcc.PC_COMM_CIVIL_SOC_LEAD_ROW_MST_ID:       
        name = '10.5 Community and civil society leadership, including young people'    

    elif mst_ID == pcc.PC_LEGAL_POLICY_REFORMS_ROW_MST_ID:       
        name = '10.6 Legal, policy reforms'    

    elif mst_ID == pcc.PC_HIV_OTHER_HEADER_ROW_MST_ID:                 
        name = '11. Other'  

    elif mst_ID == pcc.PC_HIV_DBL_CLICK_ADD_OTHER_ITEMS_ROW_MST_ID:    
        name = '11.1 Other items'

    return name

def get_PC_row_string_constant(mst_ID):
    string_constant = ''

    # Standard rows
    if mst_ID == pcc.PC_PROG_SPEC_HR_HEADER_ROW_MST_ID:
        string_constant = 'GB_stprogSpecHR'

    elif mst_ID == pcc.PC_NAT_LEVEL_STAFF_ROW_MST_ID:
        string_constant = 'GB_stNatStaff'

    elif mst_ID == pcc.PC_REG_LEVEL_STAFF_ROW_MST_ID:
        string_constant = 'GB_stISProvinStaff'

    elif mst_ID == pcc.PC_DIST_LEVEL_STAFF_ROW_MST_ID:
        string_constant = 'GB_stDistStaff'

    elif mst_ID == pcc.PC_TRAIN_HEADER_ROW_MST_ID:
        string_constant = 'GB_stTraining'

    elif mst_ID == pcc.PC_INSERV_REFRESH_TRAIN_ROW_MST_ID:
        string_constant = 'GB_stInServRefTrain'

    elif mst_ID == pcc.PC_TRAIN_OF_TRAIN_ROW_MST_ID:
        string_constant = 'GB_stTrainOfTrainers'

    elif mst_ID == pcc.PC_DEV_TRAIN_PROG_MAT_ROW_MST_ID:
        string_constant = 'GB_stDevTrainProgAndMat'

    elif mst_ID == pcc.PC_CHANGE_PRESERV_TRAIN_CURRIC_ROW_MST_ID:
        string_constant = 'GB_stChangePreServTrain'

    elif mst_ID == pcc.PC_SUPP_ACT_ROW_MST_ID:
        string_constant = 'GB_stSuppActivities'

    elif mst_ID == pcc.PC_SUPERVIS_HEADER_ROW_MST_ID:
        string_constant = 'GB_stSupervision'

    elif mst_ID == pcc.PC_COORD_MEET_ROW_MST_ID:
        string_constant = 'GB_stCoordMeets'

    elif mst_ID == pcc.PC_NAT_STAFF_VISIT_LOCAL_ROW_MST_ID:
        string_constant = 'GB_stNatStaffVisLocal'

    elif mst_ID == pcc.PC_MONITOR_EVAL_HEADER_ROW_MST_ID:
        string_constant = 'GB_stMonitAndEval'

    elif mst_ID == pcc.PC_DESIGN_ME_FRAME_SYS_ROW_MST_ID:
        string_constant = 'GB_stDesMEFramewrkSys'

    elif mst_ID == pcc.PC_DESIGN_QC_QA_ROW_MST_ID:
        string_constant = 'GB_stDesQCAssur'

    elif mst_ID == pcc.PC_DESIGN_REV_DATA_MGMT_SYS_ROW_MST_ID:
        string_constant = 'GB_stDesRevDataMgmt'
 
    elif mst_ID == pcc.PC_DATA_COLLECT_ANALYSIS_ROW_MST_ID:
        string_constant = 'GB_stDataCollectAna'

    elif mst_ID == pcc.PC_QC_QA_ROW_MST_ID:
        string_constant = 'GB_stMonitQualityControl'

    elif mst_ID == pcc.PC_INFRA_EQUIP_HEADER_ROW_MST_ID:
        string_constant = 'GB_stInfraEquip'

    elif mst_ID == pcc.PC_SIT_ASSESS_INFRA_ROW_MST_ID:
        string_constant = 'GB_stInfraSituationalAsses'

    elif mst_ID == pcc.PC_EQUIP_UPGRADES_LOW_TIER_ROW_MST_ID:
        string_constant = 'GB_stEquipUpgradesLowTierFac'

    elif mst_ID == pcc.PC_EQUIP_UPGRADES_HOSP_ROW_MST_ID:
        string_constant = 'GB_stEquipUpgradesHospitals'

    elif mst_ID == pcc.PC_TRANSPORT_HEADER_ROW_MST_ID:
        string_constant = 'SM_stTransport'

    elif mst_ID == pcc.PC_SIT_ASSESS_TRANSPORT_ROW_MST_ID:
        string_constant = 'GB_stTransSituationalAsses'

    elif mst_ID == pcc.PC_NEW_VEHIC_PURCHASE_ROW_MST_ID:
        string_constant = 'GB_stNewVehicPurchase'

    elif mst_ID == pcc.PC_VEHIC_OP_MAINT_ROW_MST_ID:
        string_constant = 'GB_stVehicOpMaint'

    elif mst_ID == pcc.PC_COMM_MEDIA_HEADER_ROW_MST_ID:
        string_constant = 'GB_stCommMediaOutrch'

    elif mst_ID == pcc.PC_DEV_COMM_STRAT_ROW_MST_ID:
        string_constant = 'GB_stDevCommStrat'

    elif mst_ID == pcc.PC_MASS_MEDIA_COMM_ROW_MST_ID:
        string_constant = 'GB_stMassMedia'

    elif mst_ID == pcc.PC_PRINTED_MATS_ROW_MST_ID:
        string_constant = 'GB_stPrintedMats'

    elif mst_ID == pcc.PC_SOCIAL_OUTR_ACTIV_ROW_MST_ID:
        string_constant = 'GB_stSocialOutreachActiv'

    elif mst_ID == pcc.PC_ADVOC_HEADER_ROW_MST_ID:
        string_constant = 'GB_stAdvocacy'

    elif mst_ID == pcc.PC_PLAN_ADVOC_STRAT_ROW_MST_ID:
        string_constant = 'GB_stPannAdvocacyStrat'

    elif mst_ID == pcc.PC_ADVOC_ACTS_ROW_MST_ID:
        string_constant = 'GB_stAdvocacyActiv'

    elif mst_ID == pcc.PC_ADVOC_MATS_ROW_MST_ID:
        string_constant = 'GB_stAdvocacyMats'

    elif mst_ID == pcc.PC_GEN_PROG_MGMT_HEADER_ROW_MST_ID:
        string_constant = 'GB_stGenProgMgmt'

    elif mst_ID == pcc.PC_DESIGN_REV_COUNTRY_STRAT_ROW_MST_ID:
        string_constant = 'GB_stDesRevCntyStrat'

    elif mst_ID == pcc.PC_DEV_REVIEW_ANNUAL_WORKPLAN_ROW_MST_ID:
        string_constant = 'GB_stDevRevAnnWrkPlan'

    elif mst_ID == pcc.PC_DEV_REVIEW_HR_PLAN_ROW_MST_ID:
        string_constant = 'GB_stDevRevHRPlan'

    elif mst_ID == pcc.PC_PROG_COORD_MEET_ROW_MST_ID:
        string_constant = 'GB_stProgCoordMeet'

    elif mst_ID == pcc.PC_COMMOD_REG_POL_ROW_MST_ID:
        string_constant = 'GB_stCommodityRegPolicyNoNum'

    elif mst_ID == pcc.PC_SIT_ANALYSIS_ROW_MST_ID:
        string_constant = 'GB_stPCSitAnaCoordGrp'

    elif mst_ID == pcc.PC_OTHER_HEADER_ROW_MST_ID:
        string_constant = 'GB_stOther'

    elif mst_ID == pcc.PC_DBL_CLICK_ADD_OTHER_ITEMS_ROW_MST_ID:
        string_constant = 'GB_stAddOtherItems'

    elif mst_ID == pcc.PC_OFFICE_EQUIP_SUPP_ROW_MST_ID:
        string_constant = 'GB_stOfficeEquipSupplies'
        
    elif mst_ID == pcc.PC_UTIL_ROW_MST_ID:
        string_constant = 'GB_stUtilities'

    # HIV-specific rows

    elif mst_ID == pcc.PC_POLICY_BASED_INTERV_ROW_MST_ID:
        string_constant = 'GB_stPolicyBasedInterventions'

    elif mst_ID == pcc.PC_SOC_ECON_SUPP_CHILD_ROW_MST_ID:
        string_constant = 'GB_stSocioeconSuppChildren'

    elif mst_ID == pcc.PC_PREV_VIOL_WOMEN_ROW_MST_ID:           
        string_constant = 'GB_stPrevViolenceAgainstWomen'       

    elif mst_ID == pcc.PC_DECRIM_KEY_POP_BEHAV_ROW_MST_ID:      
        string_constant = 'GB_stDecrimKeyPopBehav'           

    elif mst_ID == pcc.PC_PREV_STIGMA_DISCRIM_ROW_MST_ID:       
        string_constant = 'GB_stPrevStigmaDiscrim'    

    elif mst_ID == pcc.PC_COMM_CIVIL_SOC_LEAD_ROW_MST_ID:       
        string_constant = 'GB_stCommCivilSocLead'    

    elif mst_ID == pcc.PC_LEGAL_POLICY_REFORMS_ROW_MST_ID:       
        string_constant = 'GB_stLegalPolicyReforms'    

    elif mst_ID == pcc.PC_HIV_OTHER_HEADER_ROW_MST_ID:                 
        string_constant = 'GB_stOther'  

    elif mst_ID == pcc.PC_HIV_DBL_CLICK_ADD_OTHER_ITEMS_ROW_MST_ID:    
        string_constant = 'GB_stAddOtherItems'

    return string_constant

def get_PC_row_is_aggregate(mst_ID):
    # HIV programme area has different rows at the end, but the master IDs don't overlap with the standard programme areas, so we can utilize
    # the same function.
    if (mst_ID == pcc.PC_PROG_SPEC_HR_HEADER_ROW_MST_ID) \
        or (mst_ID == pcc.PC_TRAIN_HEADER_ROW_MST_ID) \
        or (mst_ID == pcc.PC_SUPERVIS_HEADER_ROW_MST_ID) \
        or (mst_ID == pcc.PC_MONITOR_EVAL_HEADER_ROW_MST_ID) \
        or (mst_ID == pcc.PC_INFRA_EQUIP_HEADER_ROW_MST_ID) \
        or (mst_ID == pcc.PC_TRANSPORT_HEADER_ROW_MST_ID) \
        or (mst_ID == pcc.PC_COMM_MEDIA_HEADER_ROW_MST_ID) \
        or (mst_ID == pcc.PC_ADVOC_HEADER_ROW_MST_ID) \
        or (mst_ID == pcc.PC_GEN_PROG_MGMT_HEADER_ROW_MST_ID) \
        or (mst_ID == pcc.PC_OTHER_HEADER_ROW_MST_ID) \
        or (mst_ID == pcc.PC_HIV_OTHER_HEADER_ROW_MST_ID):
            return True
    else:
        return False

def get_PC_row_has_data(prog_area_ID: str, mst_ID: str):
    if (prog_area_ID == pccl.PC_IH_HIV_AIDS_MST_ID) and (mst_ID == pcc.PC_POLICY_BASED_INTERV_ROW_MST_ID):
        return False
    else:
        return True

def get_PC_row_is_cost(prog_area_ID: str, mst_ID: str):
    if (prog_area_ID == pccl.PC_IH_HIV_AIDS_MST_ID) and (mst_ID == pcc.PC_POLICY_BASED_INTERV_ROW_MST_ID):
        return False
    else:
        return True

def get_PC_row_is_single_line(prog_area_ID: str, mst_ID: str):
    if ((prog_area_ID == pccl.PC_IH_HIV_AIDS_MST_ID) and ((mst_ID == pcc.PC_SOC_ECON_SUPP_CHILD_ROW_MST_ID) 
        or (mst_ID == pcc.PC_PREV_VIOL_WOMEN_ROW_MST_ID) 
        or (mst_ID == pcc.PC_DECRIM_KEY_POP_BEHAV_ROW_MST_ID) 
        or (mst_ID == pcc.PC_PREV_STIGMA_DISCRIM_ROW_MST_ID) 
        or (mst_ID == pcc.PC_PREV_STIGMA_DISCRIM_ROW_MST_ID) 
        or (mst_ID == pcc.PC_COMM_CIVIL_SOC_LEAD_ROW_MST_ID) 
        or (mst_ID == pcc.PC_LEGAL_POLICY_REFORMS_ROW_MST_ID))):
            return True
    else:
        return False

def get_PPLI_intensity_value(phase_ID):
    intensity = 0

    if phase_ID == pcc.PC_EARLY_IMP_PHASE_25_PPLI_MST_ID:
        intensity = 25

    elif phase_ID == pcc.PC_PARTIAL_IMP_PHASE_50_PPLI_MST_ID:
        intensity = 50

    elif phase_ID == pcc.PC_FULL_IMP_PHASE_100_PPLI_MST_ID:
        intensity = 100

    return intensity

def get_PPLI_phase_curr_ID(phase_ID):
    phase_curr_ID = gbc.GB_NOT_FOUND

    if phase_ID == pcc.PC_NO_ACTION_PHASE_0_PPLI_MST_ID:
        phase_curr_ID = pcc.PC_NO_ACTION_PHASE_0_PPLI_CURR_ID

    elif phase_ID == pcc.PC_PLANNING_PHASE_0_PPLI_MST_ID:
        phase_curr_ID = pcc.PC_PLANNING_PHASE_0_PPLI_CURR_ID

    elif phase_ID == pcc.PC_EARLY_IMP_PHASE_25_PPLI_MST_ID:
        phase_curr_ID = pcc.PC_EARLY_IMP_PHASE_25_PPLI_CURR_ID

    elif phase_ID == pcc.PC_PARTIAL_IMP_PHASE_50_PPLI_MST_ID:
        phase_curr_ID = pcc.PC_PARTIAL_IMP_PHASE_50_PPLI_CURR_ID

    elif phase_ID == pcc.PC_FULL_IMP_PHASE_100_PPLI_MST_ID:
        phase_curr_ID = pcc.PC_FULL_IMP_PHASE_100_PPLI_CURR_ID

    return phase_curr_ID

def PC_get_health_sys_admin_unit_ID(admin_level_curr_ID):
    ID = str(gbc.GB_NOT_FOUND)

    if admin_level_curr_ID == pcc.PC_PPLI_NATIONAL_ADMIN_LEVEL_CURR_ID:
        ID = str(pccl.PC_IH_NATIONAL_HSAU_MST_ID)

    elif admin_level_curr_ID == pcc.PC_PPLI_PROVINCE_ADMIN_LEVEL_CURR_ID:
        ID = str(pccl.PC_IH_PROVINCE_HSAU_MST_ID)

    elif admin_level_curr_ID == pcc.PC_PPLI_DISTRICT_ADMIN_LEVEL_CURR_ID:
        ID = str(pccl.PC_IH_DISTRICT_HSAU_MST_ID)

    return ID




from datetime import date
from copy import deepcopy

import numpy as np




import SpectrumCommon.Const.RP.RPConst as rpc
import SpectrumCommon.Const.RP.RPDatabaseConst as rpdbc
import SpectrumCommon.Const.RP.RPConstLink as rpcl
import SpectrumCommon.Const.RP.RPTags as rpmvt
import SpectrumCommon.Modvars.RP.RPModVarDefs as rpmvd
# import SpectrumCommon.Util.RP.RPUtilLink as pcul

import SpectrumCommon.Modvars.RP.RPReadDatabases as rprd

def init_modvars(country_ID, first_year, final_year, extra):
    '''
        Initializes RAPID MVs for new projections.

            Parameters:
                countryID (string):  Ex: BEN for Benin.

                first_year (int): First year selected by user.

                final_year (int): Final year selected by user. 

                extra (dict): Any other fields.  
 
            Returns:
                mod_vars_rec (dict): MVs.
    '''
    
    # modules = extra[gbc.GB_MODULES]
    # sector_on_MV = extra[gbc.GB_RAPID_SECTORS_ON]
    active_methods_MV = extra[rpcl.RP_FP_ACTIVE_METHODS_TAG]
    method_mix_MV = extra[rpcl.RP_FP_METHOD_MIX_TAG]

    # Load databases
    country_defaults_DB = rprd.RP_load_country_defaults_DB(country_ID)
    contraceptive_use_med_DB = rprd.RP_load_contraceptive_use_med_DB(country_ID)
    contraceptive_use_acc_DB = rprd.RP_load_contraceptive_use_acc_DB(country_ID)
    country_defaults_sources_DB = rprd.RP_load_country_defaults_sources_DB(country_ID)
    urban_DB = rprd.RP_load_urban_DB(country_ID)
    SSP1_DB = rprd.RP_load_SSP1_DB(country_ID)
    SSP2_DB = rprd.RP_load_SSP2_DB(country_ID)
    SSP3_DB = rprd.RP_load_SSP3_DB(country_ID)
    SSP5_DB = rprd.RP_load_SSP5_DB(country_ID)

    value_bool = False
    value_int = 0
    value_flt = 0.0
    value_str = ''
    value_date = date.today().strftime("%B %d, %Y"),   
    use_lists = True
    include_status_quo = True

    num_years_inputs = final_year - first_year + 1
    num_years_outputs = final_year - first_year + 1

    indirect_prog_costs = country_defaults_DB[rpdbc.RP_COST_INDIRECTS_KEY_CDDB]

    # sector_records = rpmvd.RP_init_sector_records(sector_on_MV, num_years_inputs)
    sector_records = rpmvd.RP_init_sector_records(num_years_inputs)
    rprd.RP_set_sector_defaults(country_defaults_DB, urban_DB, sector_records, first_year, final_year)
    # sector_records_meta = rpmvd.RP_init_sector_records(sector_on_MV, num_years_inputs, True)
    sector_records_meta = rpmvd.RP_init_sector_records(num_years_inputs, True)
    rprd.RP_set_sector_defaults(country_defaults_DB, urban_DB, sector_records_meta, first_year, final_year)

    climate_params_rec = rpmvd.RP_create_climate_params_rec()
    rprd.RP_set_climate_param_defaults(country_defaults_DB, climate_params_rec)

    scenario_goal_scale_up = np.full((num_years_inputs), 0).tolist()
    scenario_counterfactual_scale_up = np.full((num_years_inputs), 0).tolist()

    FP_scen_scale_up_rec = rpmvd.RP_init_FP_scen_scale_up_rec(num_years_inputs)
    rprd.RP_set_FP_scen_scale_up_defaults(contraceptive_use_med_DB, contraceptive_use_acc_DB, 
        FP_scen_scale_up_rec, scenario_goal_scale_up, scenario_counterfactual_scale_up, first_year, final_year)

    avg_annual_unit_direct_costs = np.full((rpcl.RP_FP_NUM_METHODS + 1), 0.0).tolist()
    rprd.RP_set_FP_method_cost_defaults(country_defaults_DB, active_methods_MV, avg_annual_unit_direct_costs)

    perc_users = np.full((rpcl.RP_FP_NUM_METHODS + 1), 0.0).tolist()

    SSP_data = rprd.RP_set_SSP_data(first_year, final_year, SSP1_DB, SSP2_DB, SSP3_DB, SSP5_DB)

    # Get the sum of the active modern FamPlan method using the method mix.
    modern_methods_sum = 0.0
    for m, value in enumerate(active_methods_MV):
        if value and (m not in rpcl.RP_FP_M_TRAD_METHODS):
            modern_methods_sum += method_mix_MV[rpcl.RP_FP_ALL_AGES][m][rpcl.RP_FP_ALL_NEED][0]

    for m, value in enumerate(active_methods_MV):
        if value and (m not in rpcl.RP_FP_M_TRAD_METHODS):
            perc_users[m] = ((method_mix_MV[rpcl.RP_FP_ALL_AGES][m][rpcl.RP_FP_ALL_NEED][0]) / modern_methods_sum) * 100

    sources = rpmvd.RP_init_sources_rec()
    rprd.RP_set_sources_defaults(country_defaults_sources_DB, sources)

    mod_vars_rec = {            
        rpmvt.RP_VERSION_TAG                           : 1,
        rpmvt.RP_DATA_CHANGED_TAG                      : value_bool, 
        rpmvt.RP_PROJ_VALID_TAG                        : value_bool, 
        rpmvt.RP_FILE_NAME_TAG                         : value_str,                          
        rpmvt.RP_PROJ_VALID_DATE_TAG                   : value_date,    
          
        rpmvt.RP_SECTORS_ON_TAG                        : rpmvd.RP_init_sector_list(),
        rpmvt.RP_SECTOR_RECORDS_TAG                    : sector_records,

        rpmvt.RP_FP_SCEN_GOAL_TAG                      : rpc.RP_FP_SCEN_GOAL_ACCELERATE_MST_ID,
        rpmvt.RP_FP_SCEN_GOAL_SCALE_UP_TAG             : scenario_goal_scale_up,
        rpmvt.RP_FP_SCEN_COUNTERFACTUAL_TAG            : rpc.RP_FP_SCEN_CF_MEDIAN_MST_ID,
        rpmvt.RP_FP_SCEN_COUNTERFACTUAL_SCALE_UP_TAG   : scenario_counterfactual_scale_up,

        rpmvt.RP_HOUSEHOLD_SIZE_BASELINE_TAG           : country_defaults_DB[rpdbc.RP_HH_SIZE_KEY_CDDB],

        rpmvt.RP_CLIMATE_PARAMS_TAG                    : climate_params_rec,

        rpmvt.RP_FP_PERC_USERS_TAG                     : perc_users,
        # rpmvt.RP_FP_TYPICAL_USE_EFFECTIVENESS_TAG      : np.full((rpcl.RP_FP_NUM_METHODS + 1), 0.0).tolist(),
        rpmvt.RP_FP_AVG_ANNUAL_UNIT_DIRECT_COSTS_TAG   : avg_annual_unit_direct_costs,
        rpmvt.RP_FP_INDIRECT_PROG_COSTS_TAG            : indirect_prog_costs,
        rpmvt.RP_REVISED_SOURCES_TAG                   : rpmvd.RP_init_sources_rec(),

        rpmvt.RP_RES_DP_BIG_POP_TAG                    : rpmvd.RP_init_res_FP_scen_rec_by_sex_age_year(use_lists, num_years_outputs, 0.0),
        rpmvt.RP_RES_DP_TFR_TAG                        : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_NO_STATUS_QUO_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_FP_BIRTHS_TAG                     : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_NO_STATUS_QUO_LIST, num_years_outputs, 0.0),

        rpmvt.RP_RES_PRIM_SCH_STUDENTS_TAG             : rpmvd.RP_init_res_FP_scen_rec_by_sex_year(use_lists, include_status_quo, num_years_outputs, 0.0),
        rpmvt.RP_RES_PRIM_EDU_COSTS_TAG                : rpmvd.RP_init_res_FP_scen_rec_by_sex_year(use_lists, not include_status_quo, num_years_outputs, 0.0),
        rpmvt.RP_RES_SEC_SCH_STUDENTS_TAG              : rpmvd.RP_init_res_FP_scen_rec_by_sex_year(use_lists, include_status_quo, num_years_outputs, 0.0),
        rpmvt.RP_RES_SEC_EDU_COSTS_TAG                 : rpmvd.RP_init_res_FP_scen_rec_by_sex_year(use_lists, not include_status_quo, num_years_outputs, 0.0),
        rpmvt.RP_RES_PRIM_SCH_TEACHERS_NEEDED_TAG      : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_PRIM_SCH_TEACHER_SALARY_COSTS_TAG : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_NO_STATUS_QUO_LIST, num_years_outputs, 0.0),

        rpmvt.RP_RES_SKILLED_DELIV_TAG                 : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_SKILLED_DELIV_COSTS_TAG           : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_NO_STATUS_QUO_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_PREG_W_4_PLUS_ANC_VISITS_TAG      : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_COST_4_PLUS_ANC_VISITS_TAG        : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_NO_STATUS_QUO_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_HLTH_PROF_TAG                     : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_HLTH_PROF_SALARY_COSTS_TAG        : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_NO_STATUS_QUO_LIST, num_years_outputs, 0.0),

        rpmvt.RP_RES_PEOPLE_ACCESS_IMRP_WATER_SOURCES_TAG           : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_ADD_PEOPLE_NEED_ACCESS_IMPR_WATER_CUMU_TAG     : rpmvd.RP_init_res_FP_scen_rec(include_status_quo, 0.0),
        rpmvt.RP_RES_CAP_COST_ADD_PEOPLE_GAIN_ACCESS_IMPR_WATER_TAG : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_NO_STATUS_QUO_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_PEOPLE_ACCESS_IMPR_SANI_FAC_TAG                : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_ADD_PEOPLE_ACCESS_IMPR_SANI_FAC_CUMU_TAG       : rpmvd.RP_init_res_FP_scen_rec(include_status_quo, 0.0),
        rpmvt.RP_RES_CAP_COST_ADD_PEOPLE_GAIN_ACCESS_IMPR_SANI_TAG  : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_NO_STATUS_QUO_LIST, num_years_outputs, 0.0),

        rpmvt.RP_RES_MAIN_STAPLE_CROP_PROD_NEEDED_TAG               : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_HECTARES_CROP_NEEDED_MEET_GOAL_TAG             : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_NO_STATUS_QUO_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_PERC_CROP_NEEDED_MEET_GOAL_TAG                 : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_NO_STATUS_QUO_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_COST_MAIN_STAPLE_CROP_PROD_TAG                 : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_NO_STATUS_QUO_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_FOOD_NEEDED_RED_FOOD_INSEC_CUMU_TAG            : rpmvd.RP_init_res_FP_scen_rec(not include_status_quo, 0.0),
        rpmvt.RP_RES_COST_PROV_FOOD_AID_PEOPLE_FOOD_INSEC_ELIM_TAG  : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_NO_STATUS_QUO_LIST, num_years_outputs, 0.0),

        rpmvt.RP_RES_URBAN_POP_TAG                                  : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_NO_STATUS_QUO_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_URBAN_HHS_LIVING_INF_SETTLEMENTS_TAG           : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_HOUSES_TO_BUILD_CUMU_TAG                       : rpmvd.RP_init_res_FP_scen_rec(include_status_quo, 0.0),
        rpmvt.RP_RES_COST_BUILD_HOUSES_PEOPLE_RELOC_TAG             : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_NO_STATUS_QUO_LIST, num_years_outputs, 0.0),

        rpmvt.RP_RES_POP_ACCESS_ELEC_TAG                            : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_ADD_PEOPLE_NEED_ACCESS_ELEC_CUMU_TAG           : rpmvd.RP_init_res_FP_scen_rec(include_status_quo, 0.0),
        rpmvt.RP_RES_CAP_COST_CONNECT_ADD_HHS_ELEC_TAG              : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_NO_STATUS_QUO_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_KWH_NEEDED_SUFF_ELEC_LEVELS_TAG                : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_COST_ELEC_TAG                                  : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_NO_STATUS_QUO_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_HHS_CLEAN_COOK_FUEL_TECH_TAG                   : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_ADD_HHS_NEED_ACCESS_CLEAN_COOK_TECH_CUMU_TAG   : rpmvd.RP_init_res_FP_scen_rec(include_status_quo, 0.0),
        rpmvt.RP_RES_COST_PROV_ADD_HHS_CLEAN_COOK_FUEL_TECH_TAG     : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_NO_STATUS_QUO_LIST, num_years_outputs, 0.0),

        rpmvt.RP_RES_AGE_DEPENDENCY_RATIO_TAG                       : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_NO_COUNTERFACTUAL_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_GDP_PER_CAPITA_TAG                             : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_NO_COUNTERFACTUAL_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_TOTAL_PEOPLE_WITH_JOBS_TAG                     : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_ADD_JOBS_NEEDED_TAG                            : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_NO_STATUS_QUO_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_NEET_TAG                                       : rpmvd.RP_init_res_FP_scen_rec_by_sex_year(use_lists, True, num_years_outputs, 0.0),
        rpmvt.RP_RES_ADD_NEET_TAG                                   : rpmvd.RP_init_res_FP_scen_rec_by_sex_year(use_lists, False, num_years_outputs, 0.0),
  
        rpmvt.RP_RES_NUM_POP_WITH_INTERV_TAG                   : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_COST_POP_WITH_INTERV_TAG                  : rpmvd.RP_init_res_FP_scen_rec_by_year(use_lists, rpc.RP_FP_SCEN_NO_STATUS_QUO_LIST, num_years_outputs, 0.0),
    
        rpmvt.RP_RES_SSP_TEMP_CHANGES_FROM_BASE_TAG           : rpmvd.RP_init_res_SSP_scen_rec_by_year(use_lists, rpc.RP_SSP_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_FEWER_KWH_NEEDED_BASED_ON_FP_ACC_TAG     : rpmvd.RP_init_res_SSP_scen_rec_by_year(use_lists, rpc.RP_SSP_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_CUMU_SAVINGS_ELEC_COSTS_DUE_FP_ACC_TAG   : rpmvd.RP_init_res_SSP_cumu_rec(),
        rpmvt.RP_RES_ADD_HECTARES_SAVED_CLIMATE_COND_TAG      : rpmvd.RP_init_res_SSP_scen_rec_by_year(use_lists, rpc.RP_SSP_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_GDP_PER_CAPITA_STATUS_QUO_TAG            : rpmvd.RP_init_res_SSP_scen_rec_by_year(use_lists, rpc.RP_SSP_BASE_LIST, num_years_outputs, 0.0),
        rpmvt.RP_RES_GDP_PER_CAPITA_ACC_TAG                   : rpmvd.RP_init_res_SSP_scen_rec_by_year(use_lists, rpc.RP_SSP_BASE_LIST, num_years_outputs, 0.0),
   
        rpmvt.RP_RES_SUMMARY_TAG : rpmvd.RP_init_res_summary_rec(),
    }

    # Meta MVs

    mod_vars_rec[rpmvt.RP_SECTOR_RECORDS_META_TAG]                   = sector_records_meta

    mod_vars_rec[rpmvt.RP_FP_SCEN_GOAL_META_TAG]                     = deepcopy(mod_vars_rec[rpmvt.RP_FP_SCEN_GOAL_TAG])
    mod_vars_rec[rpmvt.RP_FP_SCEN_COUNTERFACTUAL_META_TAG]           = deepcopy(mod_vars_rec[rpmvt.RP_FP_SCEN_COUNTERFACTUAL_TAG])
    mod_vars_rec[rpmvt.RP_FP_SCEN_SCALE_UP_META_TAG]                 = FP_scen_scale_up_rec

    mod_vars_rec[rpmvt.RP_HOUSEHOLD_SIZE_BASELINE_META_TAG]          = deepcopy(mod_vars_rec[rpmvt.RP_HOUSEHOLD_SIZE_BASELINE_TAG])

    mod_vars_rec[rpmvt.RP_FP_PERC_USERS_META_TAG]                    = deepcopy(mod_vars_rec[rpmvt.RP_FP_PERC_USERS_TAG])
    # mod_vars_rec[rpmvt.RP_FP_TYPICAL_USE_EFFECTIVENESS_META_TAG]     = []
    mod_vars_rec[rpmvt.RP_FP_AVG_ANNUAL_UNIT_DIRECT_COSTS_META_TAG]  = deepcopy(mod_vars_rec[rpmvt.RP_FP_AVG_ANNUAL_UNIT_DIRECT_COSTS_TAG])
    mod_vars_rec[rpmvt.RP_FP_INDIRECT_PROG_COSTS_META_TAG]           = deepcopy(mod_vars_rec[rpmvt.RP_FP_INDIRECT_PROG_COSTS_TAG])

    mod_vars_rec[rpmvt.RP_CLIMATE_PARAMS_META_TAG]                   = deepcopy(mod_vars_rec[rpmvt.RP_CLIMATE_PARAMS_TAG])

    mod_vars_rec[rpmvt.RP_SSP_META_TAG] = SSP_data

    mod_vars_rec[rpmvt.RP_SOURCES_META_TAG] = sources

    mod_vars_rec[rpmvt.RP_RP_SOURCES_TAG] = (
        'RAPID draws on a wide range of sources for initial parameter values that can be edited by the user.  ' +
        'Please refer to the source information on each page for details on the source for specific input.'
    )

    #Debugging
    #with open('C:\Proj\IHTSampleFile.JSON', 'w') as f:
    #   json.dump(mod_vars_dict, f)

    return mod_vars_rec


#######################################################################################################
#                                                                                                     #
#                                       Cost Category DB (CCDB)                                       #
#                                                                                                     #
#######################################################################################################

PC_COST_CAT_ID_KEY_CCDB        = 'costCatID'
PC_COST_CAT_NAME_KEY_CCDB      = 'costCatName'
PC_COST_CAT_STR_CONST_KEY_CCDB = 'costCatStringConstant'

#######################################################################################################
#                                                                                                     #
#                                       Cost Input DB (CIDB)                                          #
#                                                                                                     #
#######################################################################################################

PC_COST_CAT_ID_KEY_CIDB               = 'costCatID'
PC_COST_CAT_NAME_KEY_CIDB             = 'costCatName'
PC_COST_INPUT_ID_KEY_CIDB             = 'costInputID' 
PC_COST_INPUT_NAME_KEY_CIDB           = 'costInputName'
PC_COST_INPUT_STR_CONST_KEY_CIDB      = 'costInputStringConstant'
PC_UNIT_COST_KEY_CIDB                 = 'unitCost'
PC_UNIT_OF_MEASURE_KEY_CIDB           = 'unitOfMeasure'
PC_UNIT_OF_MEASURE_STR_CONST_KEY_CIDB = 'unitOfMeasureStringConstant'
PC_NOTE_KEY_CIDB                      = 'note'
PC_SOURCE_KEY_CIDB                    = 'source'

#######################################################################################################
#                                                                                                     #
#                                       Activity Line Item DB (ALIDB)                                 #
#                                                                                                     #
#######################################################################################################

PC_AREA_TYPE_ID_KEY_ALIDB           = 'areaTypeID'
PC_AREA_TYPE_NAME_KEY_ALIDB         = 'areaTypeName'
PC_AREA_ID_KEY_ALIDB                = 'areaID'
PC_AREA_NAME_KEY_ALIDB              = 'areaName'
PC_COSTING_SECT_ID_KEY_ALIDB        = 'costingSectionID'
PC_COSTING_SECT_NAME_KEY_ALIDB      = 'costingSectionName'
PC_COSTING_SUBSECT_ID_KEY_ALIDB     = 'costingSubsectionID'
PC_COSTING_SUBSECT_NAME_KEY_ALIDB   = 'costingSubsectionName'
PC_ACT_ID_KEY_ALIDB                 = 'activityID'
PC_ACT_NAME_KEY_ALIDB               = 'activityName'
PC_COST_INPUT_ID_KEY_ALIDB          = 'costInputID'
PC_COST_INPUT_NAME_KEY_ALIDB        = 'costInputName'
PC_NUM_UNITS_KEY_ALIDB              = 'numUnits'
PC_DURATION_KEY_ALIDB               = 'duration'
PC_FREQUENCY_KEY_ALIDB              = 'frequency'

#######################################################################################################
#                                                                                                     #
#                                       PPLI Activity Line Item DB (PALIDB)                           #
#                                                                                                     #
#######################################################################################################

PC_AREA_TYPE_ID_KEY_PALIDB           = 'areaTypeID'
PC_AREA_TYPE_NAME_KEY_PALIDB         = 'areaTypeName'
PC_AREA_ID_KEY_PALIDB                = 'areaID'
PC_AREA_NAME_KEY_PALIDB              = 'areaName'
PC_COSTING_SECT_ID_KEY_PALIDB        = 'costingSectionID'
PC_COSTING_SECT_NAME_KEY_PALIDB      = 'costingSectionName'
PC_COSTING_SUBSECT_ID_KEY_PALIDB     = 'costingSubsectionID'
PC_COSTING_SUBSECT_NAME_KEY_PALIDB   = 'costingSubsectionName'
PC_ACT_ID_KEY_PALIDB                 = 'activityID'
PC_ACT_NAME_KEY_PALIDB               = 'activityName'
PC_COST_INPUT_ID_KEY_PALIDB          = 'costInputID'
PC_COST_INPUT_NAME_KEY_PALIDB        = 'costInputName'
PC_NUM_UNITS_KEY_PALIDB              = 'numUnits'
PC_DURATION_KEY_PALIDB               = 'duration'
PC_QUANTITY_KEY_PALIDB               = 'quantity'

#######################################################################################################
#                                                                                                     #
#                                       Activity DB (ADB)                                             #
#                                                                                                     #
#######################################################################################################

PC_AREA_TYPE_ID_KEY_ADB           = 'areaTypeID'
PC_AREA_TYPE_NAME_KEY_ADB         = 'areaTypeName'
PC_AREA_ID_KEY_ADB                = 'areaID'
PC_AREA_NAME_KEY_ADB              = 'areaName'
PC_COSTING_SECT_ID_KEY_ADB        = 'costingSectionID'
PC_COSTING_SECT_NAME_KEY_ADB      = 'costingSectionName'
PC_COSTING_SUBSECT_ID_KEY_ADB     = 'costingSubsectionID'
PC_COSTING_SUBSECT_NAME_KEY_ADB   = 'costingSubsectionName'
PC_ACT_ID_KEY_ADB                 = 'activityID'
PC_ACT_NAME_KEY_ADB               = 'activityName'
PC_ACT_STR_CONST_KEY_ADB          = 'activityStringConstant'

#######################################################################################################
#                                                                                                     #
#                                       Costing Section DB (CSDB)                                     #
#                                                                                                     #
#######################################################################################################

PC_AREA_TYPE_ID_KEY_CSDB              = 'areaTypeID'
PC_AREA_TYPE_NAME_KEY_CSDB            = 'areaTypeName'
PC_AREA_ID_KEY_CSDB                   = 'areaID'
PC_AREA_NAME_KEY_CSDB                 = 'areaName'
PC_COSTING_SECT_ID_KEY_CSDB           = 'costingSectionID'
PC_COSTING_SECT_NAME_KEY_CSDB         = 'costingSectionName'
PC_COSTING_SECT_STR_CONST_KEY_CSDB    = 'costingSectionStringConstant'
PC_COSTING_SUBSECT_ID_KEY_CSDB        = 'costingSubsectionID'
PC_COSTING_SUBSECT_NAME_KEY_CSDB      = 'costingSubsectionName'
PC_COSTING_SUBSECT_STR_CONST_KEY_CSDB = 'costingSubsectionStringConstant'
PC_COSTS_CS_KEY_CSDB                  = 'costsCS'


#######################################################################################################
#                                                                                                     #
#                                       PPLI Phase DB (PPLIPDB)                                       #
#                                                                                                     #
#######################################################################################################

PC_ISO_ALPHA_3_COUNTRY_CODE_KEY_PPLIPDB  = 'ISO3AlphaCountryCode'
PC_PHASE_INTENSITY_RECORDS_KEY_PPLIPDB   = 'phaseIntensityRecords'
PC_AREA_TYPE_ID_KEY_PPLIPDB              = 'areaTypeID'
PC_AREA_ID_KEY_PPLIPDB                   = 'areaID'
PC_COSTING_SECT_ID_KEY_PPLIPDB           = 'costingSectionID'
PC_COSTING_SUBSECT_ID_KEY_PPLIPDB        = 'costingSubsectionID'
PC_PHASE_ID_KEY_PPLIPDB                  = 'phaseID'

#######################################################################################################
#                                                                                                     #
#                                       PPLI Taxation DB (PPLITDB)                                    #
#                                                                                                     #
#######################################################################################################

PC_ISO_ALPHA_3_COUNTRY_CODE_KEY_PPLITDB  = 'ISO3AlphaCountryCode'
PC_PRICE_ELASTICITY_PARAMS_KEY_PPLITDB   = 'priceElasticityParams'




#######################################################################################################
#                                                                                                     #
#                                       Staff Salaries DB (SSDB)                                      #
#                                                                                                     #
#######################################################################################################

HW_ISO_ALPHA_3_COUNTRY_CODE_KEY_SSDB = 'ISO3AlphaCountryCode'
HW_INTERVENTION_LIST_KEY_SSDB        = 'interventions'

HW_INTERV_MST_ID_KEY_SSDB     = 'intervMstID'
HW_EDU_CODE_1_SALARY_KEY_SSDB = 'eduCode1Salary'
HW_EDU_CODE_2_SALARY_KEY_SSDB = 'eduCode2Salary'
HW_EDU_CODE_3_SALARY_KEY_SSDB = 'eduCode3Salary'
HW_EDU_CODE_4_SALARY_KEY_SSDB = 'eduCode4Salary'
HW_EDU_CODE_5_SALARY_KEY_SSDB = 'eduCode5Salary'


from uuid import UUID
from pydantic import BaseModel, typing
from typing import List, Optional

from AvenirCommon.Logger import log
from AvenirCommon.Util import GBRange

from SpectrumCommon.Const.GB import *
from SpectrumCommon.Const.PJ import *


def getList_sex_singleAge_year(finalYrIdx, value = 0):
    return np.full((GB_Female + 1, GB_MaxSingleAges + 1, finalYrIdx + 1), value).tolist()

# def getList_5yrAge_sex_year(finalYrIdx, value = 0):
#     return np.full((GB_MAX_AGE + 1, GB_Female + 1, finalYrIdx + 1), value).tolist()

def getList_sex_5yrAge_year(finalYrIdx, value = 0):
    return np.full((GB_Female + 1, GB_MAX_AGE + 1, finalYrIdx + 1), value).tolist()

def getList_5yrAge_year(finalYrIdx, value = 0):
    return np.full((GB_MAX_AGE + 1, finalYrIdx + 1), value).tolist()

def getList_sex_year(finalYrIdx, value = 0):
    return np.full((GB_Female + 1, finalYrIdx + 1), value).tolist()

def getList_year(finalYrIdx, value = 0):
    return np.full(finalYrIdx + 1, value).tolist()

class calcParams(BaseModel):
    force: bool = False
    runImpact: bool = False
    runTBDynamicalModel: bool = False
    useLeapfrog: bool = False
    returnTags: list = []

class TaskMerge(BaseModel):
    id: UUID
    projID : str
    force : Optional[bool] = False

extra_dict = {
    "planType" : 0,
    GB_TB_COSTING_ON : False,
    GB_IHT_ON : False,
    GB_LIST_COSTING_ON : False,
    GB_GSKMode : False,
    "TBModel": ""
}

class createProjectionParams(BaseModel):
    fileName: str = "Test Projection"
    country: str = "BEN"
    firstYear: int = None
    finalYear: int = 2030
    modules: list = [GB_DP, GB_AM]
    userID: str = ""
    extra: dict = extra_dict

class NaomiSubnationalParams(BaseModel):
    country : str = "BEN"
    subnatRegion : str



class modifyProjectionParams(BaseModel):
    fileName: Optional[str] = None
    country: Optional[str] = None
    firstYear: Optional[int] = None
    finalYear: Optional[int] = None
    # modules: Optional[list] = None
    insertModules: Optional[list] = None
    removeModules: Optional[list] = None
    reloadModules: Optional[list] = None
    flatLinePastYears: Optional[bool] = False
    flatLineFutureYears: Optional[bool] = False
    TBSubnatRegion: Optional[str] = None
    TBImpactStatistical: Optional[bool] = None
    TBImpactDynamical: Optional[bool] = None
    appID: Optional[str] = None
    useLeapfrog: Optional[bool] = None


# class filterModules(BaseModel):
#    modvars: List[modvarParams] = [dict(modvarParams())]
class filterGetModule(BaseModel):
    module: str = "DP"
    modvars: list = ["<DP_FirstYear_V1>", "<DP_FinalYear_V1>"]


class filterGetParam(BaseModel):
    projID: str = "ceea11af94244344b4f4f9643e4c2b55"
    modules: List[filterGetModule] = [dict(filterGetModule())]


class filterSetModvar(BaseModel):
    tag: str = "<DP_FirstYear_V1>"
    value: typing.Any = 1970


class filterSetModule(BaseModel):
    module: str = "DP"
    modvars: List[filterSetModvar] = [dict(filterSetModvar())]


class filterSetParam(BaseModel):
    projID: str = "ceea11af94244344b4f4f9643e4c2b55"
    modules: List[filterSetModule] = [dict(filterSetModule())]


class LoginParams(BaseModel):
    username: str = "guest"
    password: str = "123456"


class datapackTreeParam(BaseModel):
    modSet: list = [1, 4]
    simple: bool = False
    configurations: bool = True
    configurationsKeyName: str = "config"


class datapackIndicatorsParam(BaseModel):
    modID: int = 1
    mstID: int = 1
    proj: str = ""

class citationParams(BaseModel):
    projID: str = ""
    clientVersion: str = ""

class datapackChartYears(BaseModel):
    firstYearIdx: int = 0
    finalYearIdx: int = 1
    min: int = 0
    max: int = 0


class datapackParam(BaseModel):
    chartYears: dict = dict(datapackChartYears())
    indicators: list = [dict(datapackIndicatorsParam())]
    language: int = 1

class getCountryParam(BaseModel):
    version: str = GBDatabaseVersion
    modID: int = GB_GB
    ignoreSampleCountry: bool = True

class getSubnationalParam(BaseModel):
    iso: int
    version: str = GBDatabaseVersion

class proximateDeterminantsParam(BaseModel):
    version: str = 'V2'

class missOpDataParam(BaseModel):
    countryID: str


#functions used for documenting information on modvars

def gb_shape(description : str, type, has_default=True, **kwargs):
    data_shape = {}
    data_shape['description'] = description
    data_shape['type'] = type
    data_shape['hasDefault'] = has_default
    data_shape.update(kwargs)
    return data_shape

gb_str_shape = lambda d, **kwargs : gb_shape(d, 'str', **kwargs)
gb_int_shape = lambda d, **kwargs : gb_shape(d, 'int', **kwargs)
gb_bool_shape = lambda d, **kwargs : gb_shape(d, 'bool', range='True, False', **kwargs)
gb_float_shape = lambda d, **kwargs : gb_shape(d, 'float', **kwargs)
gb_percent_shape  = lambda d, **kwargs : gb_shape(d, 'float', range='0-100', **kwargs)
gb_rate_shape  = lambda d, **kwargs : gb_shape(d, 'float', range='0.0-1.0', **kwargs)


gb_year_dim = 'year'
gb_year_indices = {'firstYear':0, 'finalYear':-1}

gb_sex_dim = 'sex'
gb_sex_indices = {'Both Sexes':GB_BothSexes, "Male":GB_Male, "Female":GB_Female}

gb_5y_age_dim = '5y_age'
gb_5y_age_indices = {
    'All Ages'      : GB_AllAges,
    'Age 0 to 4'    : GB_A0_4,
    'Age 5 to 9'    : GB_A5_9,
    'Age 10 to 14'  : GB_A10_14,
    'Age 15 to 19'  : GB_A15_19,
    'Age 20 to 24'  : GB_A20_24,
    'Age 25 to 29'  : GB_A25_29,
    'Age 30 to 34'  : GB_A30_34,
    'Age 35 to 39'  : GB_A35_39,
    'Age 40 to 44'  : GB_A40_44,
    'Age 45 to 49'  : GB_A45_49,
    'Age 50 to 54'  : GB_A50_54,
    'Age 55 to 59'  : GB_A55_59,
    'Age 60 to 64'  : GB_A60_64,
    'Age 65 to 69'  : GB_A65_69,
    'Age 70 to 74'  : GB_A70_74,
    'Age 75 to 79'  : GB_A75_79,
    'Age 80+'       : GB_A80_Up,
}

gb_5y_age_adult_dim = '5y_age_adult'
gb_5y_age_adult_indices = {
    'Age 15 to 19'  : GB_A15_19,
    'Age 20 to 24'  : GB_A20_24,
    'Age 25 to 29'  : GB_A25_29,
    'Age 30 to 34'  : GB_A30_34,
    'Age 35 to 39'  : GB_A35_39,
    'Age 40 to 44'  : GB_A40_44,
    'Age 45 to 49'  : GB_A45_49,
}

gb_single_age_dim = 'single_age'
gb_single_age_indices = {
    ('Age ' + str(a)) : a for a in GBRange(0, 80)
}

gb_single_age_child_dim = 'single_age_child'
gb_single_age_child_indices = {
    ('Age ' + str(a)) : a for a in GBRange(0, 14)
}

gb_single_age_child_17_dim = 'single_age_child_17'
gb_single_age_child_17_indices = {
    ('Age ' + str(a)) : a for a in GBRange(0, 17)
}

gb_single_age_child_4_dim = 'single_age_child_4'
gb_single_age_child_4_indices = {
    ('Age ' + str(a)) : a for a in GBRange(0, 4)
}

gb_year_shape  =  lambda d, **kwargs  : gb_shape(d,'1D float', dim=[gb_year_dim], indices=[gb_year_indices], **kwargs)
gb_sex_year_shape  =  lambda d, **kwargs  : gb_shape(d, '2D float',
                                                     dim=[gb_sex_dim, gb_year_dim],
                                                     indices=[gb_sex_indices, gb_year_indices], **kwargs)
gb_sex_single_age_year_shape  =  lambda d, **kwargs  : gb_shape(d, '3D float',
                                                     dim=[gb_sex_dim, gb_single_age_dim, gb_year_dim],
                                                     indices=[gb_sex_indices, gb_single_age_indices, gb_year_indices], **kwargs)
gb_sex_5y_age_year_shape  =  lambda d, **kwargs  : gb_shape(d, '3D float',
                                                     dim=[gb_sex_dim, gb_5y_age_dim, gb_year_dim],
                                                     indices=[gb_sex_indices, gb_5y_age_indices, gb_year_indices], **kwargs)
gb_5y_age_year_shape  =  lambda d, **kwargs  : gb_shape(d, '2D float',
                                                     dim=[gb_5y_age_dim, gb_year_dim],
                                                     indices=[gb_5y_age_indices, gb_year_indices], **kwargs)

def gb_reshape_years(mv_piece, meta_mv, original_first_year, original_final_year, new_first_year, new_final_year):

    # Convert lists to numpy arrays if they aren't already numpy arrays.
    is_list = type(mv_piece)==list
    if is_list:
        mv_piece = np.array(mv_piece)

    new_num_years = new_final_year - new_first_year + 1

    # Determine the range of years to copy from the original array to the new array.
    copy_start_year = max(original_first_year, new_first_year)
    copy_end_year = min(original_final_year, new_final_year)

    original_start_index = copy_start_year - original_first_year # index of original year dimension to start copying data
    original_end_index = copy_end_year - original_first_year # index of original year dimension to stop copying data.
    new_start_index = copy_start_year - new_first_year # index of new year dimension to start pasting data
    new_end_index = copy_end_year - new_first_year # index of new year dimension to stop pasting data

    # Identify the year dimension.
    year_dim = meta_mv['dim'].index('year')

    # Create a new array with the new number of years.
    new_shape = list(mv_piece.shape) # dimensions of array (shape) as list
    new_shape[year_dim] = new_num_years # update year dimension to new number of years
    new_mv_piece = np.zeros(new_shape, dtype=mv_piece.dtype) # create new mv_piece based on shape with updated number of years and same data type as original

    # Copy the relevant data from the original array to the new array. ... = include all dimensions up to the point of the year dimension.
    new_mv_piece[..., new_start_index:new_end_index+1] = mv_piece[..., original_start_index:original_end_index+1]

    # Fill in new years beyond the original final year with the value of the original final year
    if new_final_year > original_final_year:
        final_year_value = mv_piece.take(indices=original_end_index, axis=year_dim) # Grab elements at original_end_index along the year dimension.

        for t in range(new_end_index+1, new_num_years):
            new_mv_piece[..., t] = final_year_value

    # Turn numpy type back into the list or set the new mv_piece to the original otherwise.
    mv_piece = new_mv_piece.tolist() if is_list else new_mv_piece

    return mv_piece

# original
# def gb_reshape_years(mv_piece, meta_mv, original_first_year, original_final_year, new_first_year, new_final_year):

#     first_year_offset = new_first_year - original_first_year
#     final_year_offset = new_final_year - original_final_year
#     final_year_index =  original_final_year - original_first_year

#     #temp convert list to numpy type
#     is_list = type(mv_piece)==list
#     if is_list:
#         mv_piece = np.array(mv_piece)

#     axis = 0
#     pad_by_dim = []
#     for i, e in enumerate(meta_mv['dim']):
#         if e == gb_year_dim:
#             axis = i
#             pad_by_dim.append((0, final_year_offset))
#         else:
#             pad_by_dim.append((0, 0))


#     #extending final year
#     if final_year_offset>0:

#         #Reshapes an N dimensional array for years
#         pad_start = [0]*len(mv_piece.shape)
#         pad_end   = [0]*len(mv_piece.shape)
#         pad_end[axis] = final_year_offset
#         mv_piece = np.pad(mv_piece, pad_by_dim, mode='edge')


#     #extending first year or decreasing final year
#     if (first_year_offset>0) or  (new_final_year<=original_final_year):
#         mv_piece = mv_piece.take(indices=range(first_year_offset, final_year_index+1), axis=axis)

#     #turn numpy type back into the list
#     if is_list:
#         mv_piece = mv_piece.tolist()

#     return mv_piece

def gb_update_years_inner(mv_piece, meta_mv, original_first_year, original_final_year, new_first_year, new_final_year):
    # If a year dimension was specified for the shape, pass the piece of the MV off to get reshaped based on the new years.
    if gb_year_dim in meta_mv.get('dim', []):
        if len(mv_piece) > 0:
            mv_piece = gb_reshape_years(mv_piece,
                                        meta_mv,
                                        original_first_year,
                                        original_final_year,
                                        new_first_year,
                                        new_final_year)
            return mv_piece

        # If contingent=True, we want to ignore processing the piece for whatever reason, so just return it as-is.
        elif ('contingent' in meta_mv) and (meta_mv['contingent']):
            return mv_piece

        # Otherwise, the MV should be fixed in the shapes file.
        else:
            raise Exception('Year dimension specified in MV shapes file, but MV is of length 0. Either correct the problem, or add contingent=True to the MV shape.')

    elif meta_mv['type'] == 'dict':
        if 'innerShape' in meta_mv:
            for key in meta_mv['innerShape']:
                inner_meta = meta_mv['innerShape'][key]
                # Some MVs' parts may or may not be present. If they are missing and should always be there, then raise an exception.
                # If they are missing but it's ok, just skip over processing the key.
                if key in mv_piece :
                    # print(key)
                    try:
                        inner_mv = mv_piece[key]
                        mv_piece[key] = gb_update_years_inner(inner_mv, inner_meta, original_first_year, original_final_year, new_first_year, new_final_year)
                    except Exception as e:
                        raise Exception(f"Error processing key '{key}' in MV piece: {e}")


                elif ('contingent' in meta_mv) and (meta_mv['contingent']):
                    return mv_piece

                else:
                    continue # skip processing this key if it is not present in the MV piece.
                    # raise Exception('Key \'' + key + '\' specified in MV shapes file was not found in the MV. Either correct the problem, or add contingent=True to the MV shape.')
            return mv_piece
        else:
            if ('contingent' in meta_mv) and (meta_mv['contingent']):
                return mv_piece

            else:
                pass #no innershape specified so assumption is years do not exist in this MV piece.
                return mv_piece
            #     raise Exception('innerShape was not specified for dict type. Either correct the problem, or add contingent=True to the MV shape.')

    #???
    elif((meta_mv['type'] == 'dictlist') or (meta_mv['type'] == '1D dict')):
        for i, piece in enumerate(mv_piece):

            if 'type' in meta_mv['innerShape']:
                inner_meta = meta_mv['innerShape']
            else:
                inner_meta = {
                    'innerShape': meta_mv['innerShape'],
                    'type': 'dict',  # Default to dict if no type is specified.
                }
                if 'contingent' in meta_mv:
                    inner_meta['contingent'] = meta_mv['contingent']
            mv_piece[i] = gb_update_years_inner(piece, inner_meta, original_first_year, original_final_year, new_first_year, new_final_year)
        return mv_piece

    # Otherwise, no years to update; return same MV that was passed in.
    else:
        return mv_piece

def gb_update_years(projection, params, shapes):
    first_year_offset = params.firstYear - projection[PJN_FirstYearTag]
    final_year_offset = params.finalYear - projection[PJN_FinalYearTag]
    final_year_index =  projection[PJN_FinalYearTag] - projection[PJN_FirstYearTag]


    for tag in shapes:
        print(tag)
        if tag == '<IS_FacilityRecords_V2>':
            pass
        # if tag == '<CS_SBPrevByInterResults_V1>':
        #     pass
        if (tag not in projection):
            pass
            continue  # Skip if the tag is not present in the projection.

        try:
            meta_mv = shapes[tag]
            projection[tag] = gb_update_years_inner(projection[tag],
                                                    meta_mv,
                                                    projection[PJN_FirstYearTag],
                                                    projection[PJN_FinalYearTag],
                                                    params.firstYear,
                                                    params.finalYear)
        except Exception as e:
            log(f"Error updating years for tag '{tag}': {e}")
            raise Exception(f"Error updating years for tag '{tag}': {e}")
    return None

# By no means finished, but goal is to eventually spit out a shapes dict for a module given it's MVs.  Currently just prints out the MVs with their
def gb_generate_shapes_skeleton(MVs):
    for key, value in MVs.items():
        var_type = str(type(value).__name__)

        dimensions = 0
        if var_type == 'list':
            def get_list_dimensions(lst):
                if not isinstance(lst, list):
                    return 0
                if not lst: # An empty list is still 1-dimensional
                    return 1
                return 1 + get_list_dimensions(lst[0])

            def get_list_data_type(lst):
                # Base case: if lst is not a list, return its type
                if not isinstance(lst, list):
                    return type(lst).__name__
                # Recursive case: drill down into the list
                if lst: # check if the list is not empty
                    return get_list_data_type(lst[0])
                else:
                    return 'Empty list' # You can decide how to handle empty lists

            dimensions = get_list_dimensions(value)
            data_type = str(get_list_data_type(value))
            var_type += ", " + str(dimensions) + " dimensions, data type: " + data_type

        log(key + ': ' + var_type)



from .FilterUtils import *
from .MakeAuthenticatedCalls import *
from .MiddlewareUtils import *
from .RouterLoggingMiddleware import *
from .VersionUtils import *
from .ExceptionUtils import *
from .Utils import *


def setup_general_server_calls(app):
    # general version calls
    setup_version_services(app)

    # middleware
    setup_exception_handler(app)
    if getenv("AVENIR_RUN_LOCAL", "") == "":
        setup_middleware(app)



from os import environ as env

from AvenirCommon.Database import GB_get_db_json
from AvenirCommon.Util import formatCountryFName
from SpectrumCommon.Const.GB import GBCountryListDBName, GBDatabaseVersion, GB_SPECT_MOD_DATA_CONN_ENV, \
    GB_NO_COSTING_MODE, GB_IHT_COSTING_MODE, GB_TB_COSTING_MODE, GB_LIST_COSTING_MODE
from SpectrumCommon.Const.PJ import GB_LCOnTag, GB_TBCostingOnTag, GB_IHTOnTag

def getCalcYear(data):
    return data['<projectionData>']['calcYear']

def getFinalYear(data):
    return data['<projectionData>']['finalYear']

getYearIdx = lambda year, firstYear: year - firstYear 

def getYear(t, firstYear):
    return t + firstYear

def get_country_details(countryID, subNatCode = 0):
    countries = GB_get_db_json(env[GB_SPECT_MOD_DATA_CONN_ENV], 'globals', formatCountryFName(GBCountryListDBName, GBDatabaseVersion))
    for country in countries:
        if ((country['ISO3_Alpha']==countryID)): # and (country['subNatCode']==subNatCode)):
            return country
        
def get_country_ISO3Alpha(countryISO3: int):
    countries = GB_get_db_json(env[GB_SPECT_MOD_DATA_CONN_ENV], 'globals', formatCountryFName(GBCountryListDBName, GBDatabaseVersion))
    for country in countries:
        if country['ISO3_Numeric'] == int(countryISO3):
            return country['ISO3_Alpha']
    return 'notFound'
        
def get_country_ISO3Numeric(countryISO3_Alpha: str):
    countries = GB_get_db_json(env[GB_SPECT_MOD_DATA_CONN_ENV], 'globals', formatCountryFName(GBCountryListDBName, GBDatabaseVersion))
    for country in countries:
        if country['ISO3_Alpha'] == countryISO3_Alpha:
            return country['ISO3_Numeric']
    return 'notFound'
        
def get_costing_mode(modvars: dict):
    costing_mode = GB_NO_COSTING_MODE

    assert (GB_IHTOnTag in modvars) and (GB_TBCostingOnTag in modvars) and (GB_LCOnTag in modvars), 'Costing mode tag(s) missing in MVs'

    if modvars[GB_IHTOnTag]:
        costing_mode = GB_IHT_COSTING_MODE

    elif modvars[GB_TBCostingOnTag]:
        costing_mode = GB_TB_COSTING_MODE

    elif modvars[GB_LCOnTag]:
        costing_mode = GB_LIST_COSTING_MODE

    return costing_mode


from copy import deepcopy

from SpectrumCommon.Const.AM import *
from SpectrumCommon.Const.DP import *
from SpectrumCommon.Const.PJ import *
from SpectrumCommon.Modvars.DP.DPModvarDef import DPSources_Init

def DP_UpdateModvars(projection, created_projection):
    
    # old iteration of DP_DPSourcesTag was set to 0
    if type(projection[DP_DPSourcesTag]) == int:
        projection[DP_DPSourcesTag] = DPSources_Init({})

    # from module-specific to projection-wide sources
    if DP_EdDmSourceTag in projection:
        for src in projection[DP_EdDmSourceTag]:
            projection[PJN_UserSourceTag].append(src)
            
    if AM_CMRTag in projection:
        projection[DP_CMRTag] = projection[AM_CMRTag]

    created_projection[DP_InitialConditionsTag].update(projection[DP_InitialConditionsTag])
    projection[DP_InitialConditionsTag] = deepcopy(created_projection[DP_InitialConditionsTag])

    return projection

AM_CMRTag                           = '<AM_CMR_V1>'
DP_EdDmSourceTag                    = '<DP_EdDmSource_V1>'


# Contains methods for setting up IHT ModVars

import numpy as np

import SpectrumCommon.Const.GB.GBConst as gbc

import SpectrumCommon.Const.IH as ihc

import SpectrumCommon.Const.IH.IHModVarFields as ihf

#######################################################################################################
#                                                                                                     #
#                                       Programme areas and subgroups                                 #
#                                                                                                     #
#######################################################################################################

def IH_create_prog_area_rec(ID: str, name: str, string_const: str, active: bool):

    return {
        ihf.IH_PROG_AREA_ID               : ID,
        ihf.IH_PROG_AREA_NAME             : name,
        ihf.IH_SUBGROUP_STR_CONSTANT      : string_const,
        ihf.IH_PROG_AREA_SUBGROUP_RECORDS : [],
        ihf.IH_PROG_AREA_ACTIVE           : active,
        ihf.IH_PROG_AREA_CUSTOM           : False,
    }

def IH_create_prog_area_subgroup_rec(ID: str, name: str, string_const: str, active: bool):

    return {
        ihf.IH_SUBGROUP_ID           : ID,
        ihf.IH_SUBGROUP_NAME         : name,
        ihf.IH_SUBGROUP_STR_CONSTANT : string_const,
        ihf.IH_SUBGROUP_ACTIVE       : active,
        ihf.IH_SUBGROUP_CUSTOM       : False,
    }

# This creates a programme area record with only enough info in it to identify and display it in a list.
def IH_create_prog_area_rec_ID_info_only(ID: str, name: str, string_const: str):

    return {
        ihf.IH_PROG_AREA_ID               : ID,
        ihf.IH_PROG_AREA_NAME             : name,
        ihf.IH_SUBGROUP_STR_CONSTANT      : string_const,
        ihf.IH_PROG_AREA_SUBGROUP_RECORDS : [],
    }

# This creates a subgroup record with only enough info in it to identify and display it in a list.
def IH_create_prog_area_subgroup_rec_ID_info_only(ID: str, name: str, string_const: str):

    return {
        ihf.IH_SUBGROUP_ID           : ID,
        ihf.IH_SUBGROUP_NAME         : name,
        ihf.IH_SUBGROUP_STR_CONSTANT : string_const,
    }

#######################################################################################################
#                                                                                                     #
#                                       Delivery channels                                             #
#                                                                                                     #
#######################################################################################################

def IH_create_deliv_chan_rec(ID: str, name: str, string_constant: str, active: bool):

    return {
        ihf.IH_DELIV_CHAN_ID           : ID,
        ihf.IH_DELIV_CHAN_NAME         : name,
        ihf.IH_DELIV_CHAN_STR_CONSTANT : string_constant,
        ihf.IH_DELIV_CHAN_ACTIVE       : active,
        ihf.IH_DELIV_CHAN_CUSTOM       : False,
    }

def IH_init_deliv_chan_records(costing_mode = int):
    deliv_chan_dict_list = []

    if costing_mode == gbc.GB_LIST_COSTING_MODE:

        deliv_chan_dict_list.append(
            IH_create_deliv_chan_rec(
                str(ihc.IH_COMMUNITY_CHAN_MST_ID), 
                "Community", 
                "GB_stCommunity",
                True,
            )
        )

        deliv_chan_dict_list.append(
            IH_create_deliv_chan_rec(
                str(ihc.IH_OUTREACH_CHAN_MST_ID), 
                "Outreach", 
                "GB_stOutreach",
                True,
            )
        )

        # Keep name as Clnic to match desktop.
        deliv_chan_dict_list.append(
            IH_create_deliv_chan_rec(
                str(ihc.IH_GEN_OUTPATIENT_SERV_CHAN_MST_ID), 
                "Clinic", 
                "GB_stClinic",
                True,
            )
        )

        # Keep name as Hospital to match desktop.
        deliv_chan_dict_list.append(
            IH_create_deliv_chan_rec(
                str(ihc.IH_FIRST_REFERRAL_CHAN_MST_ID), 
                "Hospital", 
                "GB_stHospital",
                True,
            )
        )

    else:
        
        deliv_chan_dict_list.append(
            IH_create_deliv_chan_rec(
                str(ihc.IH_COMMUNITY_CHAN_MST_ID), 
                "Community", 
                "GB_stCommunity",
                True,
            )
        )

        deliv_chan_dict_list.append(
            IH_create_deliv_chan_rec(
                str(ihc.IH_OUTREACH_CHAN_MST_ID), 
                "Outreach", 
                "GB_stOutreach",
                True,
            )
        )

        deliv_chan_dict_list.append(
            IH_create_deliv_chan_rec(
                str(ihc.IH_PREHOSPITAL_EMERGENCY_CHAN_MST_ID), 
                "Prehospital emergency", 
                "GB_stPrehospitalEmergency",
                True,
            )
        )

        deliv_chan_dict_list.append(
            IH_create_deliv_chan_rec(
                str(ihc.IH_GEN_OUTPATIENT_SERV_CHAN_MST_ID), 
                "General outpatient services", 
                "GB_stGenOutpatientServices",
                True,
            )
        )

        deliv_chan_dict_list.append(
            IH_create_deliv_chan_rec(
                str(ihc.IH_FIRST_REFERRAL_CHAN_MST_ID), 
                "First referral", 
                "GB_stFirstReferral",
                True,
            )
        )

        deliv_chan_dict_list.append(
            IH_create_deliv_chan_rec(
                str(ihc.IH_SEC_REFERRAL_CHAN_MST_ID), 
                "Second referral and above", 
                "GB_stSecReferralAndAbove",
                True,
            )
        )

    return deliv_chan_dict_list

#######################################################################################################
#                                                                                                     #
#                                       Delivery channel packages                                     #
#                                                                                                     #
#######################################################################################################

def IH_create_deliv_chan_pack_rec(ID: str, name: str, active: bool):

    return {
        ihf.IH_DELIV_CHAN_PACK_ID     : ID,
        ihf.IH_DELIV_CHAN_PACK_NAME   : name,
        ihf.IH_DELIV_CHAN_PACK_ACTIVE : active,
        ihf.IH_DELIV_CHAN_PACK_CUSTOM : False,
    }

#######################################################################################################
#                                                                                                     #
#                                       National programmes                                           #
#                                                                                                     #
#######################################################################################################

def IH_create_nat_prog_rec(ID: str, name: str, active: bool):

    return {
        ihf.IH_NAT_PROG_ID     : ID,
        ihf.IH_NAT_PROG_NAME   : name,
        ihf.IH_NAT_PROG_ACTIVE : active,
        ihf.IH_NAT_PROG_CUSTOM : False,
    }

#######################################################################################################
#                                                                                                     #
#                                       Audit records                                                 #
#                                                                                                     #
#######################################################################################################

def IH_create_audit_rec(status_overridden: bool, comment: str):

    return {
        ihf.IH_AUDIT_STATUS_OVERRIDDEN : status_overridden,
        ihf.IH_AUDIT_COMMENT           : comment,
    }

#######################################################################################################
#                                                                                                     #
#                                            Results                                                  #
#                                                                                                     #
#######################################################################################################

def IH_init_prog_area_staff_year_arr(num_prog_areas: int, num_staff_types: int, num_years: int, value = 0.0):
    return np.full((num_prog_areas, num_staff_types, num_years), value, gbc.GB_CALC_FLOAT_TYPE)

#######################################################################################################
#                                                                                                     #
#                                    Health system administrative units                               #
#                                                                                                     #
#######################################################################################################

def IH_create_health_sys_admin_unit_rec(ID: str, name: str, string_const: str, active: bool):

    return {
        ihf.IH_HEALTH_SYS_ADMIN_UNIT_ID               : ID,
        ihf.IH_HEALTH_SYS_ADMIN_UNIT_NAME             : name,
        ihf.IH_HEALTH_SYS_ADMIN_UNIT_STR_CONSTANT     : string_const,
        ihf.IH_HEALTH_SYS_ADMIN_UNIT_ACTIVE           : active,
        ihf.IH_HEALTH_SYS_ADMIN_UNIT_CUSTOM           : False,
        ihf.IH_HEALTH_SYS_ADMIN_UNIT_NUMBER           : 0,
    }


def IH_init_health_sys_admin_unit_records():
    health_sys_admin_unit_dict_list = []

    health_sys_admin_unit_dict_list.append(
        IH_create_health_sys_admin_unit_rec(
            str(ihc.IH_NATIONAL_HSAU_MST_ID), 
            "National", 
            "GB_stNational",
            True,
        )
    )

    health_sys_admin_unit_dict_list.append(
        IH_create_health_sys_admin_unit_rec(
            str(ihc.IH_PROVINCE_HSAU_MST_ID), 
            "Province", 
            "GB_stProvince",
            True,
        )
    )

    health_sys_admin_unit_dict_list.append(
        IH_create_health_sys_admin_unit_rec(
            str(ihc.IH_DISTRICT_HSAU_MST_ID), 
            "District", 
            "GB_stDistrict",
            True,
        )
    )

    health_sys_admin_unit_dict_list.append(
        IH_create_health_sys_admin_unit_rec(
            str(ihc.IH_ADMIN_LEVEL_4_HSAU_MST_ID), 
            "Admin level 4", 
            "GB_stAdminLevel4",
            True,
        )
    )

    health_sys_admin_unit_dict_list.append(
        IH_create_health_sys_admin_unit_rec(
            str(ihc.IH_ADMIN_LEVEL_5_HSAU_MST_ID), 
            "Admin level 5", 
            "GB_stAdminLevel5",
            True,
        )
    )

    return health_sys_admin_unit_dict_list